/*
 * cluster.c - Main routine for cluster program(s)
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <cluster.h>

int	main(int argc, char *argv[])
{
	/*****************
	 * Interpret arguments.
	 */
	extern char*	optarg;
	extern int	optind;
	int		option;
	linkageFuncType*	linkageFunc;

	linkageFunc = singleLinkage;
	while ((option = getopt(argc, argv, "ha:c:m:o:p:s:")) != -1)
	{
		switch(option)
		{
		case 'h':
			printf("cluster-1.0: help\n");
			printf("Usage: cluster-1.0 [options]\n");
			printf("Options:\n");
			printf("  -h          "
			       "You are looking at it, help.\n");
			printf("  -a number   "
			       "Absolute number of links for absolute type"
			       " linkage, default is 3.\n");
			printf("  -c number   "
			       "Minimum number of vertices needed to"
			       " print cluster, default is 2.\n");
			printf("  -m method   "
			       "Linkage method, use method \"list\" to"
			       " list the available methods.\n");
			printf("  -o number   "
			       "Cluster sort order, less than zero for"
			       " largest first, zero for\n"
			       "              "
			       "no sort, greater than zero for smallest"
			       " first, default is -1.\n");
			printf("  -p percent  "
			       "Percent used for proportional linkage,"
			       " default is 50.\n");
			printf("  -s number   "
			       "Minimum score needed to accept edge,"
			       " default is 5.\n");
			exit(1);
			break;
		case 'a':
			clusterExtern->linkageAbsolute = counterAtob(optarg);
			break;
		case 'c':
			clusterExtern->minimumNodes = counterAtob(optarg);
			break;
		case 'm':
			if (strcmp(optarg, "list") == 0)
			{
				printf("cluster-1.0: linkage methods\n");
				printf("  single             "
				       "Single linkage, default.\n");
				printf("  complete           "
				       "Complete linkage.\n");
				printf("  proportional       "
				       "Proportional linkage, this can be"
				       " abbreviated as \"pro\".\n");
				printf("  absoluteOrComplete "
				       "Absolute or complete linkage, this can"
				       " be abbreviated\n"
				       "                     "
				       "as \"aoc\".\n");
				exit(1);
			}
			else if (strcmp(optarg, "single") == 0)
			{
				linkageFunc = singleLinkage;
				clusterExtern->mergeTest = 0;
				clusterExtern->deleteEdges = 0;
			}
			else if (strcmp(optarg, "complete") == 0)
			{
				linkageFunc = linkage;
				clusterExtern->mergeTest
					= mergeTestComplete;
				clusterExtern->deleteEdges
					= deleteEdgesGeneric;
			}
			else if (strcmp(optarg, "proportional") == 0
				 || strcmp(optarg, "pro") == 0)
			{
				linkageFunc = linkage;
				clusterExtern->mergeTest
					= mergeTestProportional;
				clusterExtern->deleteEdges
					= deleteEdgesGeneric;
			}
			else if (strcmp(optarg, "absoluteOrComplete") == 0
				 || strcmp(optarg, "aoc") == 0)
			{
				linkageFunc = linkage;
				clusterExtern->mergeTest
					= mergeTestAbsoluteOrComplete;
				clusterExtern->deleteEdges
					= deleteEdgesGeneric;
			}
			else
			{
				stop("cluster: bad linkage method.\n");
			}
			break;
		case 'o':
			clusterExtern->dumpOrder = atoi(optarg);
			break;
		case 'p':
			clusterExtern->linkageProportion = atoi(optarg);
			if (clusterExtern->linkageProportion < 0
			    || clusterExtern->linkageProportion > 100)
			{
				stop("cluster: proportional linkage percent"
				     " not between 0 and 100\n");
			}
			break;
		case 's':
			clusterExtern->minimumScore = scoreAtob(optarg);
			break;
		case '?':
			stop("cluster: bad arguments (options).\n");
		}
	}

#ifdef IncludeExcludeFiles
	if (optind + 2 > argc)
	{
		stop("cluster: bad arguments, missing include and exclude"
		     " file names.\n");
	}
	nextEdgeInit(argv[optind], argv[optind + 1]);
#else
	if (optind != argc)
	{
		stop("cluster: bad arguments.\n");
	}
#endif
	linkageFunc();
	exit(0);
}
