/*
 * clusterCmp.c - Cluster comparison routine that "orders" clusters.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <cluster.h>

/*****************
 **** This code needs fixing.  If the score isn't integral then the diff
 **** could be none zero but less than 1.  Need to use a scoreCmp function?
 **** What about the type of diff?  Is an int diff value sufficient for
 **** unsigned nodeId's (or even unsigned long nodeId's)?
 ****/

/*****************
 * Compare two clusters by comparing each property of a cluster until
 * a difference is found or we know the clusters are equal.  Although
 * there is technically no way for a cluster to be equal since any node
 * is only part of one cluster and there are no duplicate nodes, thus
 * no duplicate clusters.  Worst case is you end up comparing the ids of
 * the first node in the cluster which we know are different.
 */
int	clusterCmp(clusterType *a, clusterType *b)
{
	int	diff;
#if DataTypesIndex == 2
	scoreType	scoreDiff;
#endif

	/*****************
	 * Compare the number of nodes.  Larger clusters first so
	 * a and b are reversed.
	 */
	diff = b->n - a->n;
	if (diff)
		return(diff);

	/*****************
	 * Compare the lowest score (highest in tree) of each cluster.
	 * Highest score first so a and b are reversed.
	 */

#if DataTypesIndex == 2
	scoreDiff = b->root->score - a->root->score;
	if (scoreDiff == 0.0)
	{
		diff = 0;
	}
	else if (scoreDiff > 0.0)
	{
		diff = 1;
	}
	else
	{
		diff = -1;
	}
#else
	diff = b->root->score - a->root->score;
#endif

	if (diff)
		return(diff);

	/*****************
	 * Compare the id of the clusters.
	 */
#ifdef nodeCmp
	diff = nodeCmp(a->nodes->id, b->nodes->id);
#else
	diff = a->nodes->id - b->nodes->id;
#endif
	return(diff);
}
