/*
 * deleteEdges.c - Delete edges that are no longer needed.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <cluster.h>

/*****************
 * Routine to delete all edges between every node of cluster a and every node
 * of cluster b.  Make sure you order a and b since the edge between a and
 * b is only store once (upper triangular matrix).
 */

void	deleteEdgesGeneric(clusterType* aCluster, clusterType* bCluster)
{
	nodeType*	aNode;
	nodeType*	bNode;
	edgeSetType*	edgeSet;
	nodeType*	key;
	char		found;

	aNode = aCluster->nodes;
	while (aNode != 0)
	{
		bNode = bCluster->nodes;
		while (bNode != 0)
		{
			if (
#ifdef nodeCmp
			    nodeCmp(aNode->id, bNode->id) > 0
#else
			    aNode->id > bNode->id
#endif
			    )
			{
				edgeSet = &bNode->edges;
				key = aNode;
			}
			else
			{
				edgeSet = &aNode->edges;
				key = bNode;
			}

			edgeSDelete(edgeSet, (void *) key, &found);
			bNode = bNode->next;
		}
		aNode = aNode->next;
	}
}
