/*
 * input.c - Read input data from file into internal format.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <cluster.h>
/* The following is for the strtod() declaration. */
#include <math.h>
/*#include <ctype.h>
 */

#ifndef DataTypesIndex
#error DataTypesIndex must be defined, check the Makefile.
#else

#if DataTypesIndex == 1

#include <netinet/in.h>

/*****************
 * Read in binary edge data in Entrez format (Uint + Uint + Uint).
 * This is hard coded intentionally since alot of other programs will need
 * to be changed if it is changed here (printEdges, toEdges, mmqsort,
 * upTrianleEdges for starters).
 */

typedef int		ioType;
typedef unsigned int	networkType;

#ifdef IncludeExcludeFiles
int	readEntrezBin(edgeType *edge)
#else
int	nextEdge(edgeType* edge)
#endif
{

	/*****************
	 * This code assumes a int is 4 bytes long.
	 */
	ioType			inputs[3];

	if (fread((char *) inputs, sizeof(ioType), 3, clusterExtern->edgeFile)
	    != 3)
	{
		if (ferror(clusterExtern->edgeFile))
			stop("readEntrezBin: read error\n");
		return(0);
	}
	edge->a = (ioType) ntohl((networkType) inputs[0]);
	edge->b = (ioType) ntohl((networkType) inputs[1]);
	edge->score = (ioType) ntohl((networkType) inputs[2]);

	return(1);
}

#elif DataTypesIndex == 2

#ifdef IncludeExcludeFiles
int	someFunctionWithANewNameThatShouldNotBeNeeded(edgeType *edge)
#else
int	nextEdge(edgeType* edge)
#endif
{

#define LargeBuffer	256

	char	input[LargeBuffer];
	char *	p;

	if (fgets(input, sizeof(input), clusterExtern->edgeFile) == 0)
	{
		if (ferror(clusterExtern->edgeFile))
			stop("nextEdge: read error\n");
		return(0);
	}
	p = input;

	/* The following calls to strchr() pretty much assume nodes are
	 * strings, but we know this is true since DataTypesIndex is 2.
	 * Need to check that the nodeId's do not contain parentheses,
	 * otherwise the output file will be nonsensical.
	 */

	edge->a = nodeIdStrtob(p, &p);
	if (strchr(edge->a, ')') || strchr(edge->a, '(')) {
	    sprintf(input, "nextEdge: Name '%s' contains a '(' or ')'\n",
		    edge->a);
	    stop(input);
	}

	edge->b = nodeIdStrtob(p, &p);
	if (strchr(edge->b, ')') || strchr(edge->b, '(')) {
	    sprintf(input, "nextEdge: Name '%s' contains a '(' or ')'\n",
		    edge->b);
	    stop(input);
	}

	edge->score = scoreStrtob(p, 0);

	return(1);
}

#else	/* DataTypesIndex not 1 or 2 */
#error Bad DataTypesIndex, check the Makefile.
#endif

#endif /* DataTypesIndex was defined. */
