/*
 * linkage.c - Bring in next edge and link it in, possibly as new cluster.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

/*****************
 * General linkage routine for linkage methods that need to store edges to
 * determine if clusters should be merged.  Currently this includes all methods
 * except single linkage (see singleLinkage.c).
 */
#include <cluster.h>

void	linkage(void)
{
	nodeSetType	nodeSetBuf, *nodeSet = &nodeSetBuf;
	nodeMemberType	*m;
	edgeType	edgeBuf, *edge = &edgeBuf;
	nodeType	*a, *b;
	nodeType*	tmp;
	char		found;

	/*****************
	 * Process all the edges.
	 */
	nodeInit(nodeSet);
	while(nextEdge(edge)) {

		/*****************
		 * Make sure this value exceeds the cutoff.
		 */
		if (edge->score < clusterExtern->minimumScore)
			continue;

		/*****************
		 * Search or insert the nodes.
		 */
		m = nodeSInsert(nodeSet, edge->a, &found);
		if (!found) {
			a = newNode();
			a->id = edge->a;
			m->value = (void *) a;
			edgeInit(&a->edges);
		}
		else {
			a = (nodeType *) m->value;
		}
		m = nodeSInsert(nodeSet, edge->b, &found);
		if (!found) {
			b = newNode();
			b->id = edge->b;
			m->value = (void *) b;
			edgeInit(&b->edges);
		}
		else {
			b = (nodeType *) m->value;
		}

		/*****************
		 * Merge the two nodes only if they aren't already merged.
		 */
		if (a->memberOf != b->memberOf)
		{
			/*****************
			 * Make sure a is before b and store this edge.
			 */
			if (
#ifdef nodeCmp
			    nodeCmp(a->id, b->id) > 0
#else
			    a->id > b->id
#endif
			    )
			{
				tmp = a;
				a = b;
				b = tmp;
			}
			edgeSInsert(&a->edges, (void *) b, &found);

			/*****************
			 * Check if these clusters should be merged.
			 */
			if (clusterExtern->mergeTest(a->memberOf,
						     b->memberOf,
						     edge)
			    != 0)
			{
				merge(a->memberOf, b->memberOf, edge);
			}
		}
	}
#if 0
	nodeDump(nodeSet);
#endif
	doDump();
}
