/*
 * merge.c - Merge two clusters.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <cluster.h>

void	merge(clusterType *a, clusterType *b, edgeType *edge)
{
	clusterType	*tmp;
	linkType	*link;
	nodeType	*node;

	/*****************
	 * Make an effort to put the tree in conanical form by "alphabetizing"
	 * the clusters.  This also guarantees that a is the larger cluster.
	 */
	if (clusterCmp(a, b) > 0) {
		tmp = a;
		a = b;
		b = tmp;
	}

	/*****************
	 * Delete the edges if necessary.
	 */
	if (clusterExtern->deleteEdges != 0)
	{
		clusterExtern->deleteEdges(a, b);
	}

	/*****************
	 * Link the two link trees with a new link.
	 */
	link = (linkType *) malloc(sizeof(linkType));
	if (!link)
		stop("merge: malloc failed\n");
	link->left = a->root;
	link->right = b->root;
	link->score = edge->score;
#ifdef OutputFullLink
	link->a = edge->a;
	link->b = edge->b;
#endif /* OutputFullLink */

	/*****************
	 * Merge the nodes of the clusters.
	 */
	a->lastNode->next = b->nodes;
	a->lastNode = b->lastNode;
	a->n += b->n;
	a->root = link;

	/*****************
	 * Unlink b from the list of clusters.  Since the list is circular,
	 * and has a nil, we can just set the two pointers.
	 */
	b->next->prev = b->prev;
	b->prev->next = b->next;

	/* Reduce the count by one. */
	clusterExtern->n--;

	/*****************
	 * Change all the memberOf fields of b cluster nodes to a.
	 */
	node = b->nodes;
	do {
		node->memberOf = a;
		node = node->next;
	} while (node);
	free((char *) b);
}
