/*
 * newNode.c - Allocate and initialize a new node.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <cluster.h>

nodeType	*newNode(void)
{
	clusterType	*cluster;
	linkType	*link;
	nodeType	*node;

	/*****************
	 * Allocate one of each type of object.
	 */
	node = (nodeType *) malloc(sizeof(nodeType));
	cluster = (clusterType *) malloc(sizeof(clusterType));
	link = (linkType *) malloc(sizeof(linkType));
	if (! (node && cluster && link))
		stop("newNode: malloc failed\n");

	/*****************
	 * Now link up all the pointers.  Since a cluster with corresponding
	 * links and nodes is a circular pseudo-list, it is sufficient to
	 * just return the node pointer and everything else can be indirectly
	 * accessed.
	 */

	clusterExtern->nNodes++;
	/* Have the calling routine assign the node->id for convenience. */
	node->memberOf = cluster;
	node->next = 0;
	/* Have the node->edges set to be initialize by the calling routine
	 * since singleLinkage() doesn't need to maintain a list of edges. */
	cluster->nodes = node;
	cluster->lastNode = node;
	cluster->n = 1;
	cluster->root = link;
	/* Make this the last cluster. */
	cluster->prev = clusterExtern->nil->prev;
	clusterExtern->nil->prev = cluster;
	cluster->next = clusterExtern->nil;
	cluster->prev->next = cluster;
	clusterExtern->n++;

	link->left = 0;		/* NULL link->left implies node link. */
	link->right = (linkType *) node;
	link->score = 0;	/* Is this ideal for float scores? */
	return(node);
}
