/*
 * output.c - Write out internal format data.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved worldwide.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <cluster.h>

int	dumpCmp(clusterType **a, clusterType **b);
int	dumpCmpReverse(clusterType **a, clusterType **b);
void	dumpTree(linkType *link);

/*****************
 * Print out all the clusters.
 */
void	doDump(void)
{
	unsigned int	size;
	counterType	count;
	clusterType	**sortCluster, **sortClusters;
	clusterType	*cluster;

	/*****************
	 * Allocate memory for array of pointers to cluster pointers.
	 */
	size = (clusterExtern->n + 1) * sizeof(*sortCluster);
	sortClusters = (clusterType **) malloc(size);
	if (!sortClusters)
		stop("doDump: malloc failed\n");

	/*****************
	 * Filter out clusters below the minimum size.
	 */
	for (cluster = clusterExtern->nil->next,
	     sortCluster = sortClusters; cluster != clusterExtern->nil;
	     cluster = cluster->next)
	{
		if (cluster->n >= clusterExtern->minimumNodes)
			*sortCluster++ = cluster;
	}
	*sortCluster = 0;
	count = sortCluster - sortClusters;

	/*****************
	 * Printout the statistics.
	 */
	fprintf(stderr, "Printing %d clusters from %d nodes ...\n",
		count, clusterExtern->nNodes);

	if (clusterExtern->dumpOrder > 0)
		qsort((void *) sortClusters, count, sizeof(sortCluster),
		      (qsortFuncType) dumpCmp);
	else if (clusterExtern->dumpOrder < 0)
		qsort((void *) sortClusters, count, sizeof(sortCluster),
		      (qsortFuncType) dumpCmpReverse);
	for (sortCluster = sortClusters; *sortCluster; sortCluster++) {
		dumpTree((*sortCluster)->root);
		putc('\n', clusterExtern->clusterFile);
	}
	free((char *) sortClusters);
}

void	dumpTree(linkType *link)
{
	/*****************
	 * Check if this is a node or true link.
	 */
	if (link->left) {
		putc('(', clusterExtern->clusterFile);
		dumpTree(link->left);
#ifdef OutputFullLink
		fprintf(clusterExtern->clusterFile,
			" " scoreFormat
			"," nodeIdFormat
			"," nodeIdFormat
			" ",
			link->score,
			link->a,
			link->b);
#else /* OutputFullLink */
/****
		fprintf(clusterExtern->clusterFile,
			" " scoreFormat " ",
			link->score);
 ****/
		fprintf(clusterExtern->clusterFile,
			" %0.2f ",
			link->score);
#endif /* OutputFullLink */
		dumpTree(link->right);
		putc(')', clusterExtern->clusterFile);
	}
	else {			/* This is a node. */
		fprintf(clusterExtern->clusterFile,
			nodeIdFormat,
			((nodeType *) link->right) -> id);
	}
}

/*****************
 * The following are rather simple comparison functions, they just
 * need to dereference the pointer to a cluster pointer.
 */
int	dumpCmp(clusterType **a, clusterType **b)
{
	int	diff;
#if DataTypesIndex == 2
	scoreType	scoreDiff;
#endif

	diff = (*a)->n - (*b)->n;
	if (diff)
		return(diff);

	/* Note that the comparison is reversed so the highest score
	 * is first. */
#if DataTypesIndex == 2
	scoreDiff = (*b)->root->score - (*a)->root->score;
	if (scoreDiff == 0.0)
	{
		diff = 0;
	}
	else if (scoreDiff > 0.0)
	{
		diff = 1;
	}
	else
	{
		diff = -1;
	}
#else
	diff = (*b)->root->score - (*a)->root->score;
#endif
	if (diff)
		return(diff);

#ifdef nodeCmp
	diff = nodeCmp((*a)->nodes->id, (*b)->nodes->id);
#else
	diff = (*a)->nodes->id - (*b)->nodes->id;
#endif
	return(diff);
}

int	dumpCmpReverse(clusterType **a, clusterType **b)
{
	int	diff;
#if DataTypesIndex == 2
	scoreType	scoreDiff;
#endif

	diff = (*b)->n - (*a)->n;
	if (diff)
		return(diff);

	/* Note that the comparison is reversed only because highest score
	 * first is wanted, not because this is a reverse comparison. */
#if DataTypesIndex == 2
	scoreDiff = (*b)->root->score - (*a)->root->score;
	if (scoreDiff == 0.0)
	{
		diff = 0;
	}
	else if (scoreDiff > 0.0)
	{
		diff = 1;
	}
	else
	{
		diff = -1;
	}
#else
	diff = (*b)->root->score - (*a)->root->score;
#endif
	if (diff)
		return(diff);

	/* Note that the comparsion of nodeId's shouldn't be reversed. */
#ifdef nodeCmp
	diff = nodeCmp((*a)->nodes->id, (*b)->nodes->id);
#else
	diff = (*a)->nodes->id - (*b)->nodes->id;
#endif
	return(diff);
}
