/*
 * cSet.h - Header file for C dynamic set library.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

/*****************
 * This is the header file for the C Set library that provides an
 * implementation of dynamic sets using various data structures.
 * Only common definitions and such are stored here.
 */

#ifndef cSetDefined
#define cSetDefined

typedef struct cSetExtern cSetExtern;

struct cSetExtern {
	/* Memory management routines, part of the cType
	 *  library. */
	char	*(*malloc)();
	char	*(*realloc)();
	void	(*free)();
	/* Exit like function. */
	int	(*abort)();
};

#if defined(cSetVariables) && !defined(cSetVariablesDefined)
#define cSetVariablesDefined

/*****************
 * Need to include actual external data buffer once.
 */
extern char	*_cTypeMalloc(unsigned size);
extern char	*_cTypeRealloc(char *buf, unsigned size);
/* extern void	free(char *buf); */
extern int	_cSetExit(int status);

char	_cSetBuffer[128];
cSetExtern cSetExternV = {
	_cTypeMalloc,		/* malloc */
	_cTypeRealloc,		/* realloc */
	free,			/* free */
	_cSetExit,		/* abort */
	};
cSetExtern *cSetExternP = &cSetExternV;

#else /* cSetVariables */

extern cSetExtern	*cSetExternP;

#endif /* cSetVariables */

/*****************
 * This file will automatically have some parameter definitions added
 * to the end of it along with the final endif.
 */
#define cbStypeNum	1
#define crStypeNum	2

#endif /* cSetDefined */
