/*
 * cType.h - Header file for C type library.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

/*****************
 * This is the include file for the C data type library called cType.
 */

#ifndef cTypeDefined
#define cTypeDefined
#include <config.h>
#include <ctype.h>

typedef struct cTypeExtern cTypeExtern;

struct cTypeExtern {
	/* Memory management routines, established primarily for cSet
	 *  library which makes use of this cType library. */
	char	*(*malloc)();
	char	*(*realloc)();
	void	(*free)();
	/* Exit like function. */
	int	(*abort)();
};

/* extern void	free(char *buf); */

#if defined(cTypeVariables) && !defined(cTypeVariablesDefined)
#define cTypeVariablesDefined

/*****************
 * Need to include actual external data buffer once.
 */
extern char	*_cTypeMalloc(unsigned size);
extern char	*_cTypeRealloc(char *buf, unsigned size);
extern int	_cTypeExit(int status);

cTypeExtern cTypeExternV = {
	_cTypeMalloc,		/* malloc */
	_cTypeRealloc,		/* realloc */
	free,			/* free */
	_cTypeExit,		/* abort */
	};
cTypeExtern *cTypeExternP = &cTypeExternV;

#else /* cTypeVariables */

extern cTypeExtern	*cTypeExternP;

#endif /* cTypeVariables */

/*****************
 * Prototypes or macros.  This file has a correspondence to the file
 * ../perl/cTypeSet.ph, so if a new type is added it should be put here.
 */

/* v -> void */
#ifndef cvTstrtob
#define cvTstrtob(p, pp)	0
#endif
#ifndef cvTatob
#define cvTatob(p)		0
#endif
#ifndef cvTfree
#define cvTfree(v)		0
#endif
#ifndef cvTformat
#define cvTformat		"%d"
#endif
#ifndef cvTconv
#define cvTconv(v)		0
#endif

/* p -> void *  */
#ifndef cpTstrtob
#define cpTstrtob(p, pp)	((void *) strtol(p, pp, 0))
#endif
#ifndef cpTatob
#define cpTatob(p)		((void *) atol(p))
#endif
#ifndef cpTfree
#define cpTfree(v)		0
#endif
#ifndef cpTformat
#define cpTformat		"%lu"
#endif
#ifndef cpTconv
#define cpTconv(v)		((unsigned long) v)
#endif

/* s -> char *  */
char	*csTstrtob(char *p, char **pp);
#ifndef csTatob
#define csTatob(p)		csTstrtob(p, 0);
#endif
#ifndef csTfree
#define csTfree(v)		free(v);
#endif
#ifndef csTformat
#define csTformat		"%s"
#endif
#ifndef csTconv
#define csTconv(v)		(v)
#endif

/* f -> float  */
#ifndef cfTstrtob
#define cfTstrtob(p, pp)	((float) strtod(p, pp))
#endif
#ifndef cfTatob
#define cfTatob(p)		((float) atof(p))
#endif
#ifndef cfTfree
#define cfTfree(v)		0
#endif
#ifndef cfTformat
#define cfTformat		"%g"
#endif
#ifndef cfTconv
#define cfTconv(v)		(v)
#endif

/* d -> double  */
#ifndef cdTstrtob
#define cdTstrtob(p, pp)	strtod(p, pp)
#endif
#ifndef cdTatob
#define cdTatob(p)		atof(p)
#endif
#ifndef cdTfree
#define cdTfree(v)		0
#endif
#ifndef cdTformat
#define cdTformat		"%g"
#endif
#ifndef cdTconv
#define cdTconv(v)		(v)
#endif

/* e -> long double  */
/* NEEDS SOME WORK */
#ifndef ceTstrtob
#define ceTstrtob(p, pp)	((long double) strtod(p, pp))
#endif
#ifndef ceTatob
#define ceTatob(p)		((long double) atof(p))
#endif
#ifndef ceTfree
#define ceTfree(v)		0
#endif
#ifndef ceTformat
#define ceTformat		"%g"
#endif
#ifndef ceTconv
#define ceTconv(v)		((double) v)
#endif

/* c -> char  */
#ifndef ccTstrtob
#define ccTstrtob(p, pp)	((pp? *pp = p + 1: 0), *p)
#endif
#ifndef ccTatob
#define ccTatob(p)		(*p)
#endif
#ifndef ccTfree
#define ccTfree(v)		0
#endif
#ifndef ccTformat
#define ccTformat		"%c"
#endif
#ifndef ccTconv
#define ccTconv(v)		(v)
#endif

/* b -> signed char  */
#ifndef cbTstrtob
#define cbTstrtob(p, pp)	((signed char) ((pp? *pp = p + 1: 0), *p))
#endif
#ifndef cbTatob
#define cbTatob(p)		((signed char) *p)
#endif
#ifndef cbTfree
#define cbTfree(v)		0
#endif
#ifndef cbTformat
#define cbTformat		"%d"
#endif
#ifndef cbTconv
#define cbTconv(v)		((int) v)
#endif

/* C -> unsigned char  */
#ifndef cCTstrtob
#define cCTstrtob(p, pp)	((unsigned char) ((pp? *pp = p + 1: 0), *p))
#endif
#ifndef cCTatob
#define cCTatob(p)		((unsigned char) *p)
#endif
#ifndef cCTfree
#define cCTfree(v)		0
#endif
#ifndef cCTformat
#define cCTformat		"%u"
#endif
#ifndef cCTconv
#define cCTconv(v)		((unsigned) v)
#endif

/* h -> short  */
#ifndef chTstrtob
#define chTstrtob(p, pp)	((short) strtol(p, pp, 0))
#endif
#ifndef chTatob
#define chTatob(p)		((short) atoi(p))
#endif
#ifndef chTfree
#define chTfree(v)		0
#endif
#ifndef chTformat
#define chTformat		"%d"
#endif
#ifndef chTconv
#define chTconv(v)		((int) v)
#endif

/* H -> unsigned short  */
#ifndef cHTstrtob
#define cHTstrtob(p, pp)	((unsigned short) strtol(p, pp, 0))
#endif
#ifndef cHTatob
#define cHTatob(p)		((unsigned short) atoi(p))
#endif
#ifndef cHTfree
#define cHTfree(v)		0
#endif
#ifndef cHTformat
#define cHTformat		"%u"
#endif
#ifndef cHTconv
#define cHTconv(v)		((unsigned) v)
#endif

/* i -> int  */
#ifndef ciTstrtob
#define ciTstrtob(p, pp)	((int) strtol(p, pp, 0))
#endif
#ifndef ciTatob
#define ciTatob(p)		atoi(p)
#endif
#ifndef ciTfree
#define ciTfree(v)		0
#endif
#ifndef ciTformat
#define ciTformat		"%d"
#endif
#ifndef ciTconv
#define ciTconv(v)		(v)
#endif

/* I -> unsigned int  */
#ifndef cITstrtob
#define cITstrtob(p, pp)	((unsigned int) strtol(p, pp, 0))
#endif
#ifndef cITatob
#define cITatob(p)		((unsigned int) atol(p))
#endif
#ifndef cITfree
#define cITfree(v)		0
#endif
#ifndef cITformat
#define cITformat		"%u"
#endif
#ifndef cITconv
#define cITconv(v)		(v)
#endif

/* l -> long  */
#ifndef clTstrtob
#define clTstrtob(p, pp)	strtol(p, pp, 0)
#endif
#ifndef clTatob
#define clTatob(p)		atol(p)
#endif
#ifndef clTfree
#define clTfree(v)		0
#endif
#ifndef clTformat
#define clTformat		"%ld"
#endif
#ifndef clTconv
#define clTconv(v)		(v)
#endif

/* L -> unsigned long  */
#ifndef cLTstrtob
#define cLTstrtob(p, pp)	((unsigned long) strtol(p, pp, 0))
#endif
#ifndef cLTatob
#define cLTatob(p)		((unsigned long) atol(p))
#endif
#ifndef cLTfree
#define cLTfree(v)		0
#endif
#ifndef cLTformat
#define cLTformat		"%lu"
#endif
#ifndef cLTconv
#define cLTconv(v)		(v)
#endif

/* m -> long long  */
/* NEEDS SOME WORK */
#ifndef cmTstrtob
#define cmTstrtob(p, pp)	((long long) strtol(p, pp, 0))
#endif
#ifndef cmTatob
#define cmTatob(p)		((long long) atol(p))
#endif
#ifndef cmTfree
#define cmTfree(v)		0
#endif
#ifndef cmTformat
#define cmTformat		"%ld"
#endif
#ifndef cmTconv
#define cmTconv(v)		((long) v)
#endif

/* M -> unsigned long long */
/* NEEDS SOME WORK */
#ifndef cMTstrtob
#define cMTstrtob(p, pp)	((unsigned long long) strtol(p, pp, 0))
#endif
#ifndef cMTatob
#define cMTatob(p)		((unsigned long long) atol(p))
#endif
#ifndef cMTfree
#define cMTfree(v)		0
#endif
#ifndef cMTformat
#define cMTformat		"%lu"
#endif
#ifndef cMTconv
#define cMTconv(v)		((unsigned long) v)
#endif

/*****************
 * This file will automatically have some parameter definitions added
 * to the end of it along with the final endif.
 */
#define cvTtypeNum	1
#define cpTtypeNum	2
#define csTtypeNum	3
#define cfTtypeNum	4
#define cdTtypeNum	5
#define ceTtypeNum	6
#define cCTtypeNum	7
#define cHTtypeNum	8
#define cITtypeNum	9
#define cLTtypeNum	10
#define cMTtypeNum	11
#define ccTtypeNum	12
#define cbTtypeNum	13
#define chTtypeNum	14
#define ciTtypeNum	15
#define clTtypeNum	16
#define cmTtypeNum	17

#endif /* cTypeDefined */
