/*
 * crpIvvS.c - Method of C dynamic set library.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

/*****************
 * DO NOT EDIT this file.  This file was automatically generated by the
 * command "../../perl/cSetTran.pl ../../c/cSet/rt.c" and any changes
 * should be made to the translation command itself or the source file that
 * was translated instead of directly to this file.
 */

#include <stdio.h>
#include <cSet/crpIvvS.h>

/*
 * rt.c - C dynamic set routines used by both r.c and t.c
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

/*****************
 * This file, rt.c is shared by the "r" and "t" set types.
 *
 * For a discussion about the concepts behind this code by see Section III
 * on Data Structures, p. 197 of:
 *
 *	"Introduction to Algorithms" by Thomas H. Cormen, Charles E. Leiserson,
 *	and Ronald L. Rivest; 1990 MIT Press ISBN 0-262-03141-8 or
 *	McGraw-Hill ISBN 0-07-013143-0
 *
 * Although any reference discussing red-black (2-3) trees should be
 * helpful.
 */

/* Internal function definitions not explicitly declared as static. */
void	crpIvvSpostFree(crpIvvSset *S, crpIvvSmember *P);

/*****************
 * Initialize a set.
 */
void	crpIvvSinit(crpIvvSset *S)
{
	/* Allocate a member for nil and initialize. The values
	 * of left, right, and parent for S->nil need to point somewhere
	 * reasonable.  Several ("r") routines; insertFixup, deleteFixup,
	 * leftRotate, and rightRotate, will set the parent of S->nil and
	 * care should be taken to reinitialize when necessary.  Additionally,
	 * the key value is often set to
	 * the current key and then is assumed to contain a reasonable value
	 * to save a comparison operation when searching.
	 */
	S->nil = (crpIvvSmember *) cSetExternP->malloc(sizeof(crpIvvSmember));
	S->nil->left = S->nil;
	S->nil->right = S->nil;
	S->nil->parent = S->nil;
#ifdef cSetR
	S->nil->black = 1;
#endif /* cSetR */

	/* Point the root to nil. */
	S->root = S->nil;
}

/*****************
 * Finalize a set.
 */
void	crpIvvSfinal(crpIvvSset *S)
{
	/* Free all the members by doing a post-order traversal and
	 * freeing each member. */
	if (S->root != S->nil) {
		crpIvvSpostFree(S, S->root);
	}
	cSetExternP->free(S->nil);
	S->nil = 0;
	S->root = S->nil;
}

/*****************
 * Traverse a binary tree (post-order) and free the memory.
 */
void	crpIvvSpostFree(crpIvvSset *S, crpIvvSmember *P)
{
	if (P->left != S->nil) {
		crpIvvSpostFree(S, P->left);
	}
	if (P->right != S->nil) {
		crpIvvSpostFree(S, P->right);
	}
	cSetExternP->free(P);
}

/*****************
 * Insert a member into a set.
 */
crpIvvSmember	*crpIvvSinsert(crpIvvSset *S, void * K)
{
	crpIvvSmember	*x, *y, *z;
#ifdef crpIvvScmp
	int		d = 0;
#endif /* crpIvvScmp */

	/*****************
	 * Find the place to insert this member.
	 */

	x = S->root;
	y = S->nil;
	while (x != S->nil) {
		y = x;
#ifdef crpIvvScmp
		d = crpIvvScmp(K, y->key);
		x = (d < 0)? y->left: y->right;
#else /* crpIvvScmp */
		x = (K < y->key)? y->left: y->right;
#endif /* crpIvvScmp */
	}

	/*****************
	 * Allocate and initialize a new element.
	 */
	z = (crpIvvSmember *) cSetExternP->malloc(sizeof(crpIvvSmember));
	z->key = K;
	z->left = z->right = S->nil;

	/*****************
	 * Link the new member into the tree.
	 */
	z->parent = y;
	if (y == S->nil) {
		S->root = z;
	}
	else {
		if (
#ifdef crpIvvScmp
		    d < 0
#else /* crpIvvScmp */
		    K < y->key
#endif /* crpIvvScmp */
		    ) {
			y->left = z;
		}
		else {
			y->right = z;
		}
	}

#ifdef cSetR
	crpIvvSinsertFixup(S, z);
#endif /* cSetR */

	return(z);
}

/*****************
 * Search for a member of a set.
 */
crpIvvSmember	*crpIvvSsearch(crpIvvSset *S, void * K)
{
	crpIvvSmember	*x;
#ifdef crpIvvScmp
	int		d;
#endif /* crpIvvScmp */

	x = S->root;
#ifdef crpIvvScmp
	while (x != S->nil) {
		d = crpIvvScmp(K, x->key);
		if (!d) {
			return(x);
		}
		x = (d < 0)? x->left: x->right;
	}
	return(0);
#else /* crpIvvScmp */
	S->nil->key = K;
	while (K != x->key) {
		x = (K < x->key)? x->left: x->right;
	}
	return((x == S->nil)? 0: x);
#endif /* crpIvvScmp */
}

/*****************
 * Search for member of a set or insert a member into the set.
 */
crpIvvSmember	*crpIvvSsInsert(crpIvvSset *S, void * K, char *found)
{
	crpIvvSmember	*x, *y, *z;
#ifdef crpIvvScmp
	int		d = 0;
#endif /* crpIvvScmp */

	/*****************
	 * Find the member being searched for or the place where the new
	 * member should be inserted.
	 */
	x = S->root;
	y = S->nil;
#ifdef crpIvvScmp
	while (x != S->nil) {
		y = x;
		d = crpIvvScmp(K, y->key);
		if (!d) {
			*found = 1;
			return(x);
		}
		x = (d < 0)? x->left: x->right;
	}
#else /* crpIvvScmp */
	S->nil->key = K;
	while (K != x->key) {
		y = x;
		x = (K < y->key)? x->left: x->right;
	}
	if (x != S->nil) {
		*found = 1;
		return(x);
	}
#endif /* crpIvvScmp */

	/*****************
	 * Didn't find a member with the given key, so insert it.
	 */
	*found = 0;

	/*****************
	 * Allocate and initialize a new element.
	 */
	z = (crpIvvSmember *) cSetExternP->malloc(sizeof(crpIvvSmember));
	z->key = K;
	z->left = z->right = S->nil;

	/*****************
	 * Link the new member into the tree.
	 */
	z->parent = y;
	if (y == S->nil) {
		S->root = z;
	}
	else {
		if (
#ifdef crpIvvScmp
		    d < 0
#else /* crpIvvScmp */
		    K < y->key
#endif /* crpIvvScmp */
		    ) {
			y->left = z;
		}
		else {
			y->right = z;
		}
	}

#ifdef cSetR
	crpIvvSinsertFixup(S, z);
#endif /* cSetR */

	return(z);

} /* End of function crpIvvSsInsert() */

/*****************
 * Delete a member from a set.
 */
void		crpIvvSdelete(crpIvvSset *S, crpIvvSmember *P)
{
	crpIvvSmember	*x, *y, *z;

	/* For a description of this code checkout the text mentioned above. */
	z = P;
	if (z->left == S->nil || z->right == S->nil) {
		y = z;
	}
	else {
		y = crpIvvSnext(S, z);
	}
	if (y->left != S->nil) {
		x = y->left;
	}
	else {
		x = y->right;
	}
	/* SIDE EFFECT, the following line possibly sets S->nil->parent
	 * to something other than S->nil.  I have yet to see where this
	 * presents a difficulty though.  It is expected to be this way
	 * by the "r" deleteFixup function though so don't change it here,
	 * but rather consider setting S->nil->parent to S->nil after
	 * deleteFixup is called. */
	x->parent = y->parent;
	if (y->parent == S->nil) {
		S->root = x;
	}
	else if (y == y->parent->left) {
		y->parent->left = x;
	}
	else {
		y->parent->right = x;
	}
	if (y != z) {
		z->key = y->key;
	}

#ifdef cSetR
	if (y->black) {
		crpIvvSdeleteFixup(S, x);
	}
#endif /* cSetR */
	S->nil->parent = S->nil;

	/*****************
	 * Free the memory allocated to this member.
	 */
#ifdef crpIvvScmp
	/* Indirect pointer key, free the buffer the pointer points to.
	 * The user should have freed the value memory if there was any. */
	cSetExternP->free(y->key);
#endif /* crpIvvScmp */
	cSetExternP->free(y);
}

/* Negligible difference between calling search to get a pointer and then
 * deleting with the pointer, and having a separate search&delete function.
 * Thus this search&delete function just calls search and then delete.
 * This should really be a macro, but it wouldn't be a "safe" one if it was.
 * The only reason there is no savings in doing both at once rather
 * than a search and then a delete is the parent (bidirectional) links.
 */
void		crpIvvSsDelete(crpIvvSset *S, void * K, char *found)
{
	crpIvvSmember	*x;

	x = crpIvvSsearch(S, K);
	if (! x) {
		*found = 0;
		return;
	}
	*found = 1;
	crpIvvSdelete(S, x);
}

/*****************
 * Get the first member of a set.
 */
crpIvvSmember 	*crpIvvSfirst(crpIvvSset *S)
{
	crpIvvSmember	*x;

	x = S->root;
	if (x == S->nil) {
		return(0);
	}
	while (x->left != S->nil) {
		x = x->left;
	}
	return(x);
}

/*****************
 * Get the last member of a set.
 */
crpIvvSmember 	*crpIvvSlast(crpIvvSset *S)
{
	crpIvvSmember	*x;

	x = S->root;
	if (x == S->nil) {
		return(0);
	}
	while (x->right != S->nil) {
		x = x->right;
	}
	return(x);
}

/*****************
 * Advance to the next member of a Set.
 */
crpIvvSmember 	*crpIvvSnext(crpIvvSset *S, crpIvvSmember *P)
{
	crpIvvSmember	*x, *y;

	x = P;

	/*****************
	 * If there is a right subtree, go all the way to the bottom left
	 * of it.
	 */
	if (x->right != S->nil) {
		x = x->right;
		while (x->left != S->nil) {
			x = x->left;
		}
		return(x);
	}

	/*****************
	 * Go up the tree until the first "right" parent (child is left).
	 */
	y = x->parent;
	while (y != S->nil && x == y->right) {
		x = y;
		y = y->parent;
	}
	return((y == S->nil)? 0: y);
}

/*****************
 * Return to the previous member of a Set.
 */
crpIvvSmember 	*crpIvvSprev(crpIvvSset *S, crpIvvSmember *P)
{
	crpIvvSmember	*x, *y;

	x = P;

	/*****************
	 * If there is a left subtree, go all the way to the bottom right
	 * of it.
	 */
	if (x->left != S->nil) {
		x = x->left;
		while (x->right != S->nil) {
			x = x->right;
		}
		return(x);
	}

	/*****************
	 * Go up the tree until the first "left" parent (child is right).
	 */
	y = x->parent;
	while (y != S->nil && x == y->left) {
		x = y;
		y = y->parent;
	}
	return((y == S->nil)? 0: y);
}

/*****************
 * Print routines
 */

void	crpIvvSdumpMember(crpIvvSset *S, crpIvvSmember *P)
{
	/* Watch out the value lines might get deleted. */
#if 0
    printf("%x %x (%s)", (unsigned) P, (unsigned) P->key, P->black? "black": "red");
#endif
#if 1
	printf(cpTformat
#ifdef cSetR
	       "  (%s)"
#endif
	       , cpTconv(P->key)
#ifdef cSetR
	       , P->black? "black": "red"
#endif
	       );
#endif
}
	       
void	crpIvvSinorderDump(crpIvvSset *S, crpIvvSmember *P, int depth)
{
	char	*depthStr = "    ";
	int	i;

	/* Do the right side first. */
	if (P->right != S->nil) {
		crpIvvSinorderDump(S, P->right, depth + 1);
	}

	/* Print this member inorder. */
	for (i = 0; i < depth; i++) {
		printf(depthStr);
	}
	crpIvvSdumpMember(S, P);
	printf("\n");

	/* Do the left side last. */
	if (P->left != S->nil) {
		crpIvvSinorderDump(S, P->left, depth + 1);
	}
}

void	crpIvvSdump(crpIvvSset *S)
{
	if (S->root != S->nil)
		crpIvvSinorderDump(S, S->root, 0);
}
