/*
 * crsIsvS.c - Method of C dynamic set library.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

/*****************
 * DO NOT EDIT this file.  This file was automatically generated by the
 * command "../../perl/cSetTran.pl ../../c/cSet/r.c" and any changes
 * should be made to the translation command itself or the source file that
 * was translated instead of directly to this file.
 */

#include <stdio.h>
#include <cSet/crsIsvS.h>

/*
 * r.c - C dynamic set routines for red-black tree.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

/* Internal function definitions not explicitly declared static. */
void		crsIsvSinsertFixup(crsIsvSset *S, crsIsvSmember *P);
void		crsIsvSdeleteFixup(crsIsvSset *S, crsIsvSmember *P);
void		crsIsvSleftRotate(crsIsvSset *S, crsIsvSmember *P);
void		crsIsvSrightRotate(crsIsvSset *S, crsIsvSmember *P);

/* Want to share some code with the "t" set type also, so set a flag
 * and include the source.
 */
#define cSetR
#include "crsIsvSrt.c"

void	crsIsvSinsertFixup(crsIsvSset *S, crsIsvSmember *P)
{
	crsIsvSmember	*x, *y;

	/*****************
	 * Step up the tree while we have a red parent, assuming we aren't
	 * at the root of the tree already.
	 */
	x = P;
	x->black = 0;
	while (x != S->root && ! x->parent->black) {

		/*****************
		 * Make sure the grandparent is oriented to the right of the
		 * parent (parent is left from grandparent).
		 */
		if (x->parent == x->parent->parent->left) {
			y = x->parent->parent->right;

			/*****************
			 * See if the uncle, which we know is on the right, is
			 * red also.  If so then just color the parent
			 * and uncle black, and the gradnparent red.  Move
			 * up to the grandparent and start again.
			 */
			if (! y->black) {
				x->parent->black = 1;
				y->black = 1;
				x->parent->parent->black = 0;
				x = x->parent->parent;
			}
			else { /* End of if (! y->black) */

				/*****************
				 * If necessary, do a left rotation so that
				 * both the parent and grandparent are to
				 * the right (childs to the left).
				 */
				if (x == x->parent->right) {
					x = x->parent;
					crsIsvSleftRotate(S, x);
				}
				
				/*****************
				 * This left subtree of the grandparent is
				 * red heavy so we need to rotate the tree
				 * to the right about the grandparent.
				 */
				x->parent->black = 1;
				x->parent->parent->black = 0;
				crsIsvSrightRotate(S, x->parent->parent);
			} /* End of if-else (! y->black) */
		}
		else { /* End of if (x->parent == x->parent->parent->left) */

			/*****************
			 * The following code is the same as above except
			 * the left and right are reversed.
			 */
			y = x->parent->parent->left;
			if (! y->black ) {
				x->parent->black = 1;
				y->black = 1;
				x->parent->parent->black = 0;
				x = x->parent->parent;
			}
			else { /* End of if (! y->black) */
				if (x == x->parent->left) {
					x = x->parent;
					crsIsvSrightRotate(S, x);
				}
				x->parent->black = 1;
				x->parent->parent->black = 0;
				crsIsvSleftRotate(S, x->parent->parent);
			} /* End of if-else (! y->black) */
		} /* End of if-else (x->parent == x->parent->parent->left) */
	} /* End of while (x != S->root && ! x->parent->black) */
	S->root->black = 1;
} /* End of function crsIsvSinsertFixup() */

void	crsIsvSdeleteFixup(crsIsvSset *S, crsIsvSmember *P)
{
	crsIsvSmember	*x, *w;

	/*****************
	 * Keep moving up and fixing the tree while this member is black
	 * and we aren't at the root of the tree.  Otherwise just make
	 * this red member black.
	 */
	x = P;
	while (x != S->root && x->black) {
		/*****************
		 * Check if the parent is oriented to the right (child is
		 * left).
		 */
		if (x == x->parent->left) {
			w = x->parent->right;

			/*****************
			 * See if the sibling, we know is on the right, is
			 * red.  If so then rotate to the left, switch
			 * the colors and point w to the right sibling at
			 * the same time.
			 */
			if (! w->black) {
				w->black = 1;
				x->parent->black = 0;
				crsIsvSleftRotate(S, x->parent);
				w = x->parent->right;
			}

			/*****************
			 * See if the cousins are both black.  In that
			 * case just make sure to change the parent
			 * to red so the while loop terminates, it will
			 * be colored black just before this function
			 * returns.
			 */
			if (w->left->black && w->right->black) {
				w->black = 0;
				x = x->parent;
			}
			else { /* Not both black cousins. */
				/*****************
				 * If the left cousin is red, we know the
				 * the right cousin is black, rotate
				 * to the right to make sure the right
				 * cousin is red in preparation for the
				 * next step.
				 */
				if (w->right->black) {
					w->left->black = 1;
					w->black = 0;
					crsIsvSrightRotate(S, w);
					w = x->parent->right;
				}

				/*****************
				 * This is a rather fancy rearrangement
				 * and recoloring step.  The idea is
				 * to even things out by rotating left the
				 * red right cousin and make it black.
				 */
				w->black = x->parent->black;
				x->parent->black = 1;
				w->right->black = 1;
				crsIsvSleftRotate(S, x->parent);
				x = S->root;
			} /* Checks for both black cousins. */
		}
		else { /* Parent oriented to the left. */

			/*****************
			 * This part of the code is similar to the above
			 * code except for left and right being reverse
			 * because the parent is oriented to the left now.
			 */
			w = x->parent->left;
			if (! w->black) {
				w->black = 1;
				x->parent->black = 0;
				crsIsvSrightRotate(S, x->parent);
				w = x->parent->left;
			}
			if (w->left->black && w->right->black) {
				w->black = 0;
				x = x->parent;
			}
			else { /* Not both black cousins. */
				if (w->left->black) {
					w->right->black = 1;
					w->black = 0;
					crsIsvSleftRotate(S, w);
					w = x->parent->left;
				}
				w->black = x->parent->black;
				x->parent->black = 1;
				w->left->black = 1;
				crsIsvSrightRotate(S, x->parent);
				x = S->root;
			} /* Checks for both black cousins. */
		} /* Check of parents orientation. */
	} /* Need to keep moving up the tree */
	x->black = 1;
	S->nil->parent = S->nil; /* Reset the parent of NIL to NIL. */
}

void	crsIsvSleftRotate(crsIsvSset *S, crsIsvSmember *P)
{
	crsIsvSmember	*x, *y;

	x = P;
	y = x->right;
	x->right = y->left;
	y->left->parent = x;
	y->parent = x->parent;
	if (x == S->root)
		S->root = y;
	else if (x == x->parent->left)
		x->parent->left = y;
	else
		x->parent->right = y;
	y->left = x;
	x->parent = y;
	S->nil->parent = S->nil; /* Reset the parent of NIL to NIL. */
}

void	crsIsvSrightRotate(crsIsvSset *S, crsIsvSmember *P)
{
	crsIsvSmember	*x, *y;

	x = P;
	y = x->left;
	x->left = y->right;
	y->right->parent = x;
	y->parent = x->parent;
	if (x == S->root)
		S->root = y;
	else if (x == x->parent->left)
		x->parent->left = y;
	else
		x->parent->right = y;
	y->right = x;
	x->parent = y;
	S->nil->parent = S->nil; /* Reset the parent of NIL to NIL. */
}
