#include <config.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <math.h>

/* Index for each main routine. */
#define PIMA_MSO_INDEX 1
#define PIMA_PM_INDEX 2
#define PIMA_PLS_INDEX 3

#define MAXAR 90		/* max. num. of AA + AA-group names         */
#define MAXV 20000		/* max. length of pattern seqs. = MAXV - 1  */
#define MAXH 20000		/* max. length of query seqs. = MAXH - 1    */
#define MAX_H_DIM 7000 /* max. single-sided dim of trace-back array */
#define MAXT 100		/* max. length of pattern/query title line  */

/* the length-depend. gap penalty (W2, coeff2) is reduced by W2_ATTEN for that
 * part of  a gap >= this length.
 */
#define W2_ATTEN_START 16

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef MAX
#define MAX(A,B) (A>B?A:B)
#endif

#ifndef MIN
#define MIN(A,B) (A<B?A:B)
#endif

#define MAX_GAP_LENGTH 16382 /* 2^14 */

typedef struct direction_struct
{
	unsigned short direction:2;
	unsigned short distance:14;
} DIRECTION_VALUES;
 
#define NWEST 1
#define NORTH 2
#define WEST  3

#define MAX_SEQ 2000
#define LOCAL_LOCAL

/* General parameters for the pima programs.  Most values are initialized
 * to defaults in get_responses.c (including main_index).
 */
typedef struct pima_params_struct {
    int main_index;		/* Index of main routine; mso, pm, or pls. */
    char *query_filename;
    char *pattern_filename;
    char *root_filename;	/* Output filename. */
    FILE *query_file;
    FILE *pattern_file;
    FILE *root_file;		/* Output file. */
    int sec_struct;
    int lex;
    int coeff1;			/* Length independent gap penalty (gap open) */
    int coeff2;			/* Length dependent gap penalty (gap extend) */
    int coeff3;			/* Seconday structure gap penalty. */
    int min_local_score;	/* Min. local score to trace next quadrant. */
    int min_local_width;	/* Min. width of quadrant to do local align. */
    char *ss_chars;
    int n_AA;			/* Number of amino acids in class/weight mat.*/
    char AA_chars[MAXAR];	/* Amino acid / pattern characters. */
    char C[MAXAR][MAXAR];	/* AA class matrix. */
    int S[MAXAR][MAXAR];	/* Weight matrix. */
    float W2_atten;		/* Reduces coeff2 after W2_ATTEN_START len. */
    char *pattern_name;		/* Pattern name. */
    char *name_suffix;		/* Suffix for sequence/pattern names. */
} pima_params_t;

extern char *program_name;

/* Function prototypes. */
void read_matrix(FILE *fp, char *AA_chars, char (*C)[MAXAR], int (*S)[MAXAR], int *n_AA, float *f_min_local_score, float *f_coeff1, float *f_coeff2);
void check_calloc(void *var_p, char *var_name);
void setup_seq(char *seq, char *name, char *Sseq, int nS, int coeff1, int coeff2, int *p_n, int **point, int **coeff1g, int **coeff2g);
void change_index(char *seq, int *len);

#ifdef LOCAL_LOCAL
void paths(int *Vpoint, int *Hpoint, int **path1, int *p_l1, int **path2, int *p_l2, int nV, int nH, int (*S)[MAXAR], int *start, long int *coeff2A, int *p_Hmax, int *coeffg1V, int *coeffg2V, int *coeffg1H, int *coeffg2H, int coeffg1, int coeff_s, char *Vseq, char *Hseq, int mso, int *p_l1a, int UPi, int UPj, int LOi, int LOj, int *p_LOLOi, int *p_LOLOj, DIRECTION_VALUES **trace_back, int V_start, int V_end, int H_start, int H_end, int min_local_score, int min_local_width);
#else
void paths(int *Vpoint, int *Hpoint, int **path1, int *p_l1, int **path2, int *p_l2, int nV, int nH, int (*S)[MAXAR], int *start, long int *coeff2A, int *p_Hmax, int *coeffg1V, int *coeffg2V, int *coeffg1H, int *coeffg2H, int coeffg1, int coeff_s, char *Vseq, char *Hseq, int mso, int *p_l1a);
#endif

void make_W2_array(int coeff2, long int *coeff2A, int array_len, float W2_atten);
void get_string_or_default(char *dfault, char *string);
float get_float_or_default(float dfalut);
void SetUp(int *point, char *seq, int n, char *Sseq, int nS, char *seq_name);
void make_gapmat(char *seq, int n, int coeff3, int coeff4, int *coeffg1, int *coeffg2);
