/*
 * format.c - Determine the file format of a name or abbreviation.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <config.h>
#include <sequ.h>

/* Return the first file format that matches an abbreviation.
 */

sequ_format_t
sequ_abbrev_to_format(char *abbreviation)
{
    if (abbreviation == NULL)
	goto RETURN_UNKNOWN;

    switch(*abbreviation) {
    case 't':
	if (strncmp(abbreviation, "tbl", strlen(abbreviation)) == CMP_EQUAL)
	    return(sequ_format_tbl);
	break;
    case 'f':
	if (strncmp(abbreviation, "fasta", strlen(abbreviation)) == CMP_EQUAL)
	    return(sequ_format_fasta);
	break;
    case 'i':
	if (strncmp(abbreviation, "ig", strlen(abbreviation)) == CMP_EQUAL)
	    return(sequ_format_ig);
	break;
    case 'g':
	if (strncmp(abbreviation, "gb", strlen(abbreviation)) == CMP_EQUAL)
	    return(sequ_format_gb);
	break;
    case 'n':
	if (strncmp(abbreviation, "nbrf", strlen(abbreviation)) == CMP_EQUAL)
	    return(sequ_format_nbrf);
	break;
    case 'e':
	if (strncmp(abbreviation, "embl", strlen(abbreviation)) == CMP_EQUAL)
	    return(sequ_format_embl);
	break;
    case 'p':
	if (strncmp(abbreviation, "pir", strlen(abbreviation)) == CMP_EQUAL)
	    return(sequ_format_pir);
	break;

	/* No default */
    }

 RETURN_UNKNOWN:
    return(sequ_format_unknown);
}

/* Return the file format with the given name.
 */

sequ_format_t
sequ_name_to_format(char *name)
{
    if (name == NULL)
	goto RETURN_UNKNOWN;

    switch(*name) {
    case 't':
	if (strcmp(name, "tbl") == CMP_EQUAL)
	    return(sequ_format_tbl);
	break;
    case 'f':
	if (strcmp(name, "fasta") == CMP_EQUAL)
	    return(sequ_format_fasta);
	break;
    case 'i':
	if (strcmp(name, "ig") == CMP_EQUAL)
	    return(sequ_format_ig);
	break;
    case 'g':
	if (strcmp(name, "gb") == CMP_EQUAL)
	    return(sequ_format_gb);
	break;
    case 'n':
	if (strcmp(name, "nbrf") == CMP_EQUAL)
	    return(sequ_format_nbrf);
	break;
    case 'e':
	if (strcmp(name, "embl") == CMP_EQUAL)
	    return(sequ_format_embl);
	break;
    case 'p':
	if (strcmp(name, "pir") == CMP_EQUAL)
	    return(sequ_format_pir);
	break;

	/* No default */
    }

 RETURN_UNKNOWN:
    return(sequ_format_unknown);
}
