/*
 * new_write.c - Create a new sequence file for writing.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <config.h>
#include <memu_macros.h>
#include <sequ.h>

/* This function creates a new sequence file, opening filename for
 * writing if fp is NULL.
 */

sequ_file_t *
sequ_new_write(char *filename, FILE *fp, sequ_format_t format)
{
    sequ_file_t *file;

    memu_new_m(file, sequ_file_t, 1);

    if (filename == NULL)
	file->filename = "noname";
    else
	file->filename = filename;

    if (format == sequ_format_unknown) {
	fprintf(stderr,
		"%s: must specify format to use for file %s\n",
		program_name,
		file->filename);
	abort();
    }

    /* open filename? */
    if (fp == NULL) {
	fp = fopen(file->filename, "w");
	if (fp == NULL) {
	    fprintf(stderr,
		    "%s: error opening file %s\n",
		    program_name,
		    file->filename);
	    perror(program_name);
	    abort();
	}
    }

    file->fp = fp;
    file->format = format;
    file->last_line = NULL;
    file->last_line_length = 0;
    file->did_first = FALSE;
    return(file);
}
