/*
 * sequ.h - Header file for sequ sequence file utility library.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#ifndef sequ_h
#define sequ_h

#ifndef NULL
#define NULL 0
#endif

#ifndef CMP_EQUAL
#define CMP_EQUAL 0
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

enum sequ_format_enum {
    sequ_format_unknown = 0,
    sequ_format_tbl,
    sequ_format_fasta,
    sequ_format_ig,
    sequ_format_gb,
    sequ_format_nbrf,
    sequ_format_embl,
    sequ_format_pir
};


struct sequ_file_struct {
    char		*filename;
    FILE		*fp;
    enum sequ_format_enum format;
    char		*last_line;
    size_t		last_line_length;
    int			did_first;
};

struct sequ_seq_struct {
    char		*id;
    char		*seq;
    size_t		length;
    char		*title;
};


typedef struct sequ_file_struct sequ_file_t;
typedef enum sequ_format_enum sequ_format_t;

typedef struct sequ_seq_struct	sequ_seq_t;


extern char *program_name;


sequ_file_t *sequ_new_read_first(sequ_seq_t **seq_p,
				    char *filename,
				    FILE *fp);
sequ_file_t *sequ_new_write(char *filename,
				 FILE *fp,
				 sequ_format_t format);
void sequ_close(sequ_file_t *file);
sequ_seq_t *sequ_read(sequ_file_t *file);
void sequ_write(sequ_file_t *file, sequ_seq_t *seq);
sequ_format_t sequ_abbrev_to_format(char *abbreviation);
sequ_format_t sequ_name_to_format(char *name);
char *sequ_format_to_name(sequ_format_t format);
char **sequ_format_names(void);
void sequ_file_free(sequ_file_t *file);
void sequ_seq_free(sequ_seq_t *seq);
void sequ_rewind(sequ_file_t *file);
void sequ_seek(sequ_file_t *file, long offset, int ptrname);
long sequ_tell(sequ_file_t *file);

/* The following are somewhat private since they are really from a different
 * library, but what the heck if somebody wants to use one of them instead
 * of including the different library also just for one of these functions.
 */
char *_sequ_getline(FILE *fp, size_t *length_p);
char *_sequ_ndup(char *str, size_t n);

#endif /* defined sequ_h */
