/**************************************************************************
                      PIMA v. 1.40

  Randall F. Smith and Temple F. Smith (1991)
  Molecular Biology Computer Research Resource
  Galleria Level 1
  Dana-Farber Cancer Institute and School of Public Health
  Harvard University
  44 Binney St., Boston MA 02115 USA
  (617)732-3746
  INTERNET: rsmith@mbcrr.harvard.edu
  BITNET: rsmith%mbcrr@husc6.bitnet

  Copyright (c) 1989,1990,1991 MBCRR, Dana-Farber Cancer Institute and
                             Harvard University

*************************************************************************/
#include <pima.h>
#include <sequ.h>
char *program_name = "pima-mso";

#define MSO

int
main(int argc, char **argv)
{
    pima_params_t params[1];
    char **Vseq,
    **Hseq,
    **Vname,
    **Hname;
    char name[10000];
    float score;
    int Hmax;
    int *coeff1gH,
     *coeff2gH,
     *coeff1gV,
     *coeff2gV;
    int nV,
      nH,
      n_Vseqs,
      n_Hseqs,
      len;
    int *Vpoint,
     *Hpoint,
      start[3],
    **path1,
    **path2,
      l1,
      l2,
      i;
    long int *coeff2A;
    char *ext;
    int aligned_len;

    sequ_file_t *query_file,
     *pat_file;
    sequ_seq_t *Vseq_seq,
     *Hseq_seq;

#ifdef LOCAL_LOCAL
    int UPi,
      UPj,
      LOi,
      LOj,
      LOLOi,
      LOLOj;
    DIRECTION_VALUES **trace_back;
    int V_start,
      V_end,
      H_start,
      H_end;

#endif

    get_responses_mso(argc, argv, params);

    pat_file = sequ_new_read_first(&Vseq_seq, "pat", params->pattern_file);
    query_file = sequ_new_read_first(&Hseq_seq, "query", params->query_file);
    sequ_rewind(pat_file);
    sequ_rewind(query_file);

    /* max gap length = 128 (determined by the size of dist_north and
     * * dist_west in DIRECTION_VALUES; see plsearch.h)
     */
    coeff2A = (long int *) calloc((unsigned) MAX(MAXV, MAXH) + 2,
				  sizeof(long int));

    check_calloc(coeff2A, "coeff2A");
    make_W2_array(params->coeff2, coeff2A, MAX(MAXV, MAXH), params->W2_atten);

    /* -- Read first input-file ----------------------------------
     * * (file format = PAT-ID <tab> SEQUENCE)
     */

    n_Vseqs = 0;
#ifndef MSO
    Vseq = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
    Vname = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));

#else
    Vseq = (char **) calloc((unsigned) 2, sizeof(char *));
    Vname = (char **) calloc((unsigned) 2, sizeof(char *));
    Hseq = (char **) calloc((unsigned) 2, sizeof(char *));
    Hname = (char **) calloc((unsigned) 2, sizeof(char *));

#endif

    while ((Vseq_seq = sequ_read(pat_file)) != NULL) {

	len = strlen(Vseq_seq->id);
	if (len < 1) {
	    fprintf(stderr, "%s: Sequence is missing a name.\n", program_name);
	    exit(1);
	}
	Vname[n_Vseqs] = (char *) calloc((unsigned) len + 2, sizeof(char));
	check_calloc((void *) Vname[n_Vseqs], "Vname in main()");
	strcpy(Vname[n_Vseqs], Vseq_seq->id);

	len = Vseq_seq->length;
	if (len < 1) {
	    fprintf(stderr, "%s: Sequence '%s' has no sequence data.\n",
		    program_name, Vseq_seq->id);
	    exit(1);
	}
	Vseq[n_Vseqs] = (char *) calloc((unsigned) len + 2, sizeof(char));
	check_calloc((void *) Vseq[n_Vseqs], "Vseq in main()");
	strcpy(Vseq[n_Vseqs], Vseq_seq->seq);

	sequ_seq_free(Vseq_seq);

	if (n_Vseqs == 0) {
	    setup_seq(Vseq[0], Vname[0], params->AA_chars, params->n_AA,
		      params->coeff1, params->coeff2,
		      &nV, &Vpoint, &coeff1gV, &coeff2gV);
	} else {
	    /* Shift seq index by 1 so that seq[1] = seq[0], etc */
	    change_index(Vseq[n_Vseqs], &len);
	}

	ext = strchr(name, '.');
	if (params->sec_struct && ext && *(ext + 1) == 's'
	    && *(ext + 2) == 's') {
	    for (i = 1; i <= len; i++) {
		if (strchr(params->ss_chars, Vseq[n_Vseqs][i]))
		    /* mark as being within ss unit by setting
		     * * gap penalty > coeff1 */
		    coeff1gV[i] = params->coeff1 + 1;
	    }
	}
	/* loop-through second input-file (query sequences) */

	n_Hseqs = 0;
#ifndef MSO
	Hseq = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
	Hname = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));

#endif

	while ((Hseq_seq = sequ_read(query_file)) != NULL) {

	    len = strlen(Hseq_seq->id);
	    if (len < 1) {
		fprintf(stderr, "%s: Sequence is missing a name.\n",
			program_name);
		exit(1);
	    }
	    Hname[n_Hseqs] = (char *) calloc((unsigned) len + 2, sizeof(char));
	    check_calloc((void *) Hname[n_Hseqs], "Hname in main()");
	    strcpy(Hname[n_Hseqs], Hseq_seq->id);

	    len = Hseq_seq->length;
	    if (len < 1) {
		fprintf(stderr, "%s: Sequence '%s' has no sequence data.\n",
			program_name, Hseq_seq->id);
		exit(1);
	    }
	    Hseq[n_Hseqs] = (char *) calloc((unsigned) len + 2, sizeof(char));
	    check_calloc((void *) Hseq[n_Hseqs], "Hseq in main()");
	    strcpy(Hseq[n_Hseqs], Hseq_seq->seq);

	    sequ_seq_free(Hseq_seq);

	    if (n_Hseqs == 0) {
		setup_seq(Hseq[0], Hname[0], params->AA_chars, params->n_AA,
			  params->coeff1, params->coeff2,
			  &nH, &Hpoint, &coeff1gH, &coeff2gH);
	    } else {
		/* Shift seq index by 1 so that seq[1] = seq[0], etc */
		change_index(Hseq[n_Hseqs], &len);
	    }

	    ext = strchr(name, '.');
	    if (params->sec_struct && ext && *(ext + 1) == 's'
		&& *(ext + 2) == 's') {
		for (i = 1; i <= len; i++) {
		    if (strchr(params->ss_chars, Hseq[n_Hseqs][i]))
			/* mark as being within ss unit by setting
			 * * gap penalty > coeff1 */
			coeff1gH[i] = params->coeff1 + 1;
		}
	    }
	    /* Find the best match score (Hmax)  */

	    if (params->lex == FALSE
		|| (params->lex == TRUE && strcmp(Vname[0], Hname[0]) < 0)) {


#ifdef LOCAL_LOCAL

		UPi = UPj = LOi = LOj = LOLOi = LOLOj = -1;

		V_start = 1;
		while (Vseq[0][V_start] == '-')
		    V_start++;
		V_end = nV;
		while (Vseq[0][V_end] == '-')
		    V_end--;
		H_start = 1;
		while (Hseq[0][H_start] == '-')
		    H_start++;
		H_end = nH;
		while (Hseq[0][H_end] == '-')
		    H_end--;


		trace_back = (DIRECTION_VALUES **) calloc((unsigned)
			 (V_end - V_start + 2), sizeof(DIRECTION_VALUES *));
		check_calloc((int *) trace_back, "trace_back");

		for (i = 0; i <= V_end - V_start + 1; i++) {
		    trace_back[i] = (DIRECTION_VALUES *) calloc((unsigned)
						      (H_end - H_start + 2),
						  sizeof(DIRECTION_VALUES));
		    check_calloc((void *) trace_back[i], "trace_back[i]");
		}
#endif

		path1 = (int **) calloc((unsigned) 2, sizeof(int *));

		check_calloc((void *) path1, "path1");
		path2 = (int **) calloc((unsigned) 2, sizeof(int *));

		check_calloc((void *) path2, "path2");
		for (i = 0; i < 2; i++) {
		    path1[i] = (int *) calloc((unsigned) nV + nH, sizeof(int));

		    check_calloc(path1[i], "path1[i]");
		    path2[i] = (int *) calloc((unsigned) nV + nH, sizeof(int));

		    check_calloc(path2[i], "path2[i]");
		}


		paths(Vpoint, Hpoint, path1, &l1, path2, &l2,
		      nV, nH, params->S, start, coeff2A, &Hmax,
		      coeff1gV, coeff2gV, coeff1gH, coeff2gH,
		      params->coeff1, params->coeff3, Vseq[0], Hseq[0], FALSE,
		      &aligned_len
#ifdef LOCAL_LOCAL
		      ,UPi, UPj, LOi, LOj, &LOLOi, &LOLOj,
		      trace_back, V_start, V_end, H_start, H_end,
		      params->min_local_score, params->min_local_width
#endif
		    );

		score = (float) (Hmax / 100.0);

		fprintf(params->root_file, "%s\t%s\t%g\t%d\n",
			Vname[0], Hname[0], score, aligned_len);
		fprintf(stdout, "%s\t%s\t%g" /* "\t%g" */ "\n",
		      Vname[0], Hname[0], score /* , score/aligned_len */ );
		fflush(params->root_file);
		fflush(stdout);


#ifndef MSO
		fprintf(stdout, "\n%f\n", score);
		print_alignment(params->C, Vpoint, Hpoint, path1, path2,
		      l1, foutp_align, Vseq, Hseq, nV, nH, n_Vseqs, n_Hseqs,
				Vname, Hname, Sname, Sext);
#endif


#ifdef LOCAL_LOCAL
		for (i = 0; i <= V_end - V_start + 1; i++)
		    free((char *) trace_back[i]);
		free((char *) trace_back);
#endif

		for (i = 0; i < 2; i++) {
		    free((char *) path1[i]);
		    free((char *) path2[i]);
		}

		free((char *) path1);
		free((char *) path2);


	    }
	    free((char *) coeff1gH);
	    free((char *) coeff2gH);
	    free((char *) Hpoint);
	    i = 0;
	    free((char *) Hseq[i]);
	    free((char *) Hname[i]);

	}

	sequ_rewind(query_file);
	free((char *) Vpoint);
	free((char *) coeff1gV);
	free((char *) coeff2gV);
	i = 0;
	free((char *) Vseq[i]);
	free((char *) Vname[i]);


    }

    free((char *) coeff2A);

    fclose(params->query_file);
    fclose(params->pattern_file);
    fclose(params->root_file);

    /* NOTREACHED */
    exit(0);
}
