/**************************************************************************
                      PLSEARCH v. 5.0 (based on PIMA v. 1.25)

  Randall F. Smith* and Temple F. Smith#

  *Human Genome Center, Dept. of Molecular and Human Genetics,
   Baylor College of Medicine
   Houston, TX  77030   USA
   713-798-4735
   rsmith@bcm.tmc.edu

  #BioMolecular Engineering Research Center, College of Engineering
   Boston Univ.
   Boston, MA    USA
   tsmith@darwin.bu.edu

  Copyright (c) 1989,1990,1991 MBCRR, Dana-Farber Cancer Institute and
                             Harvard University


*************************************************************************/
#include <plsearch.h>
#include <sequ.h>
#define PLS

char *program_name = "plsearch";

#ifdef PLS

extern void get_responses(char *program_name, int default_p, int sec_structs_p, char *query_filename, char *root_filename,
char *pattern_filename, int *p_coeff1, int *p_coeff2, int *p_coeff3, float *p_gpr_f, char *ss_chars, int *p_no_param_file,
float *p_score_cutoff, float *p_display_cutoff, FILE **file_handle, char (*C)[MAXAR], float *p_W2_atten, int *p_no_titles_file,
char *Sname, char *Sext, int *p_max_display, int *p_do_detach);

extern void print_alignment(char (*C)[MAXAR], int *Vpoint, int *Hpoint, int **path1, int **path2, int len, FILE *foutp, char **Vseq, char **Hseq, int nV, int nH, int n_Vseqs, int n_Hseqs, char **Vname, char **Hname, char *Sname, char *Sext, int (*S)[MAXAR]);

extern void get_alignment(score_record *record, FILE *finp_pat, FILE *finp_query, FILE *foutp_align, int coeff1, int coeff2, int coeff3, long int *coeff2A, int (*S)[MAXAR], char *AAseq, int nAA, int max_display, sequ_file_t *pat_file, sequ_file_t *query_file, int type);
 
extern int qsort_percent_scores (score_record *record, int n);

extern void print_hist(int *bin, FILE *foutp_align);
#endif

int
main(int argc, char **argv)
{

	FILE *finp_pat, *foutp_align;
	FILE *file_handle[12],
	    *foutp_tmp,
	    *finp_query;
	char Sseq[MAXAR],
	AAseq[MAXAR];
	char **Vseq,
	**Hseq,
	**Vname,
	**Hname,
	**Vtitle,
	**Htitle;
	char name[10000];
	float gpr_f,
	score_cutoff,
	display_cutoff,
	score;
	int coeff1,
	coeff2,
	coeff3,
	Hmax;
	int *coeff1gH,
	*coeff2gH,
	*coeff1gV,
	*coeff2gV;
	int num_scores = 0,
	no_param_file = 0,
	no_titles_file = 0;
	int nS,
	S[MAXAR][MAXAR],
	nAA,
	nV,
	nH,
	n_Vseqs,
	n_Hseqs,
	len;
	int *Vpoint,
	*Hpoint,
	start[3],
	**path1,
	**path2,
	l1,
	l2,
	i;
	char C[MAXAR][MAXAR];
	FILE *finp_weightmat;
	long int *coeff2A;
	int default_p,
	sec_structs_p,
	lex_p;
	char query_filename[60],
	output_filename[60],
	pattern_filename[60];
	char program_name[60];
	char Sname[100],
	Sext[100],
	*ext,
	ss_chars[100];
	int aligned_len;
	float W2_atten;
	FILE *foutp_raw, *foutp_percent, *foutp_aligned;

	sequ_file_t *query_file, *pat_file;
	sequ_seq_t *Vseq_seq, *Hseq_seq;
	
#ifdef PLS
	int max_pat_score, max_query_score, max_align_score;
	int max_display, min_raw_score, do_detach;
	long V_seq_ftell,
	H_seq_ftell;
	score_record *record;
	int bin_percent_score[106], bin_percent_align_score[106],
	bin_raw_score[106];
#endif

	coeff3 = 0;
	min_raw_score = MIN_RAW_SCORE * 100;

	
	/* default_p set to 1 if options are specified  */
	default_p = 0;

	argv++;
	sec_structs_p = FALSE;
	lex_p = FALSE;

#ifndef PLS
	if ((argc > 1) && !strcmp(*argv, "-s")) {
		sec_structs_p = TRUE;
		argc--;
		argv++;
	}

	if ((argc > 1) && !strcmp(*argv, "-lex")) {
		lex_p = TRUE;
		argc--;
		argv++;
	}
#endif
	if (argc == 4) {
		strcpy(query_filename, *argv);
		argv++;
		strcpy(pattern_filename, *argv);
		argv++;
		strcpy(output_filename, *argv);
		printf("query-filename: %s\n", query_filename);
		printf("database-filename: %s\n", pattern_filename);
		printf("output-filename: %s\n", output_filename);
		default_p = 1;
	} else if (argc == 3) {
		strcpy(query_filename, *argv);
		strcpy(pattern_filename, "");
		argv++;
		strcpy(output_filename, *argv);
		printf("query-filename: %s\n", query_filename);
		printf("output-filename: %s\n", output_filename);
		default_p = 1;
	} else if (argc != 1) {
		fprintf(stderr,
		    "Usage: plsearch [query-file [db-file] output-file]\n");
		exit(1);
	}
	/* all user responses are handled by the following routine;
	 * if default_p is set, all default parameters will be set
	 * and the user will not be prompted for any values  
	 */

	get_responses(		
	    /* input parameters  */
	    program_name, default_p, sec_structs_p,
	    query_filename, output_filename, pattern_filename,

	    /* returned pointers/values  */
	    &coeff1, &coeff2, &coeff3, &gpr_f, ss_chars,
	    &no_param_file, &score_cutoff, &display_cutoff,
	    file_handle, C, &W2_atten, &no_titles_file,
	    Sname, Sext,
	    &max_display, &do_detach);

	finp_pat = file_handle[1];
	foutp_tmp = file_handle[4];
	finp_query = file_handle[5];
	finp_weightmat = file_handle[8];
	foutp_raw = file_handle[9];
	foutp_percent = file_handle[10];
	foutp_aligned = file_handle[11];


	query_file = sequ_new_read_first(&Vseq_seq, "query", finp_query);
	pat_file = sequ_new_read_first(&Hseq_seq, "pat", finp_pat);
	sequ_rewind(query_file);
	sequ_rewind(pat_file);

	/* Read in similarity weight matrix  ------------------------ */
	read_weightmat(finp_weightmat, S, AAseq, &nAA);

	/* max gap length = 128 (determined by the size of dist_north and
	 * dist_west in DIRECTION_VALUES; see plsearch.h)
	 */
	coeff2A = (long int *) calloc((unsigned) MAX(MAXV, MAXH) +2, 
								  sizeof(long int));
	check_calloc(coeff2A, "coeff2A");
	make_W2_array(coeff2, coeff2A, MAX(MAXV, MAXH), W2_atten);


#ifndef PLS
	/* Read in aa-class construction matrix  -------------------- */
	read_aaclassmat(finp_aa_classmat, C, Sseq, &nS);
#endif

#ifndef PLS
	Vseq = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
	Vname = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
#else
	Vseq = (char **) calloc((unsigned) 1, sizeof(char *));
	Vname = (char **) calloc((unsigned) 1, sizeof(char *));
	Vtitle = (char **) calloc((unsigned) 1, sizeof(char *));

	Hseq = (char **) calloc((unsigned) 1, sizeof(char *));
	Hname = (char **) calloc((unsigned) 1, sizeof(char *));
	Htitle = (char **) calloc((unsigned) 1, sizeof(char *));

	record = (score_record *) calloc((unsigned) 1,
	    sizeof(score_record));
	check_calloc((int *) record, "record");
	V_seq_ftell = sequ_tell(query_file);

	for (i = 0; i <=105; i++) {
		bin_percent_score[i] = 0;
		bin_percent_align_score[i] = 0;
		bin_raw_score[i] = 0;
	}
#endif

	/* -- loop-through query sequences   */

	n_Vseqs = 0;

#if 0
	/* allow only a single query seq to be used  */
	if ((Vseq_seq = sequ_read(query_file)) != NULL) {
#else
	while ((Vseq_seq = sequ_read(query_file)) != NULL) {
#endif

		Vseq[n_Vseqs] = Vseq_seq->seq;
		Vname[n_Vseqs] = Vseq_seq->id;
		Vtitle[n_Vseqs] = Vseq_seq->title;

		if (Vseq_seq->length > MAXV)
			Vseq[0][MAXV] = '\0';

		setup_seq(Vseq[0], Vname[0], AAseq, nAA, coeff1, coeff2,
				  &nV, &Vpoint, &coeff1gV, &coeff2gV);

#ifdef PLS
		max_query_score = 0;
		for (i = 1; i <= nV; i++)	
			max_query_score += S[Vpoint[i]][Vpoint[i]];
#endif
		/* loop-through second input-file (pattern sequences) */

		n_Hseqs = 0;
#ifndef PLS
		Hseq = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
		Hname = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
#else
		H_seq_ftell = sequ_tell(pat_file);
#endif

		while ((Hseq_seq = sequ_read(pat_file)) != NULL) {

			Hseq[n_Hseqs] = Hseq_seq->seq;
			Hname[n_Hseqs] = Hseq_seq->id;
			Htitle[n_Hseqs] = Hseq_seq->title;
			
			if (Hseq_seq->length > MAXH)
				Hseq[0][MAXH] = '\0';

			setup_seq(Hseq[0], Hname[0], AAseq, nAA, coeff1, coeff2,
					  &nH, &Hpoint, &coeff1gH, &coeff2gH);

			if (nV * nH >= (MAX_H_DIM * MAX_H_DIM)) {
				if (nV <= nH) {
					if (nV >= MAX_H_DIM)
						nV = nH = MAX_H_DIM;
					else
						nH = MIN(nH, (MAX_H_DIM *MAX_H_DIM)/nV);
				} else {
					if (nH >= MAX_H_DIM)
						nV = nH = MAX_H_DIM;
					else
						nV = MIN(nV, (MAX_H_DIM * MAX_H_DIM)/nH);
				}
			}


			/* Find the best match score (Hmax)  */

			path1 = (int **) calloc((unsigned) 3, sizeof(int *));

			check_calloc((void *) path1, "path1");
			path2 = (int **) calloc((unsigned) 3, sizeof(int *));

			check_calloc((void *) path2, "path2");
			for (i = 0; i <= 2; i++) {
				path1[i] = (int *) calloc((unsigned) nV + nH, sizeof(int));

				check_calloc(path1[i], "path1[i]");
				path2[i] = (int *) calloc((unsigned) nV + nH, sizeof(int));

				check_calloc(path2[i], "path2[i]");
			}


			if (lex_p == 0 || (lex_p == 1 && strcmp(Vname[0], Hname[0]) < 0)) {

				paths(Vpoint, Hpoint, path1, &l1, path2, &l2,
				    nV, nH, S, start, coeff2A, &Hmax,
				    coeff1gV, coeff2gV, coeff1gH, coeff2gH,
				    coeff1, coeff3, Vseq[0], Hseq[0], FALSE, &aligned_len);

				score = (float) (Hmax / 100.0);
				max_pat_score = max_align_score = 0;
				for (i = 1; i <= nH; i++)
					max_pat_score += S[Hpoint[i]][Hpoint[i]];
				for (i = path1[2][l1]+1; i <= start[2]; i++)
					max_align_score += S[Hpoint[i]][Hpoint[i]];


				fprintf(foutp_tmp, "%s\t%s\t%g\t%g\t%g\t%g\t%g\t%g\n",
				    Vname[0], Hname[0], score, (float)max_query_score/100.0, 
				    (float) max_pat_score/100.0, (float) Hmax/max_pat_score,
				    (float) max_align_score/100.0, 
				    (float) Hmax/max_align_score);
				fflush(foutp_tmp);
#if 0
				fprintf(stdout, "%s\t%s\t%g\t%g\t%g\t%g\t%g\t%g\n",
				    Vname[0], Hname[0], score, (float)max_query_score/100.0, 
				    (float) max_pat_score/100.0, (float) Hmax/max_pat_score,
				    (float) max_align_score/100.0, 
				    (float) Hmax/max_align_score);
				fflush(stdout);
#endif

				if (Hmax >= min_raw_score) {				

					record = (score_record *)
						realloc((char *) record, (unsigned) (num_scores + 1) *
								sizeof(score_record));
					check_calloc ((int *) record, "record");

#if 0
					strcpy(record[num_scores].Hname, Hname[0]);
					strcpy(record[num_scores].Vname, Vname[0]);
#endif
					record[num_scores].raw_score = 
						(int) ((Hmax * 10000.0)/max_query_score);
					record[num_scores].percent_score = 
						(int) ((Hmax * 10000.0)/max_pat_score);
					record[num_scores].percent_align_score = 
						(int) ((Hmax * 10000.0)/max_align_score);
					record[num_scores].H_seq_ftell = H_seq_ftell;
					record[num_scores].V_seq_ftell = V_seq_ftell;

					bin_raw_score[(int) ((Hmax * 100.0)/max_query_score)]++;
					bin_percent_score[(int) ((Hmax * 100.0)/max_pat_score)]++;
					bin_percent_align_score[(int)((Hmax * 100.0)/max_align_score)]++;
					num_scores++;
				}
			}
#ifndef PLS
			fprintf(stdout, "\n%f\n", score);
			print_alignment(C, Vpoint, Hpoint, path1, path2,
			    l1, foutp_align, Vseq, Hseq, nV, nH, n_Vseqs, n_Hseqs,
			    Vname, Hname, Sname, Sext);
#endif
			for (i = 0; i <= 2; i++) {
				free((char *) path1[i]);
				free((char *) path2[i]);
			}

			H_seq_ftell = sequ_tell(pat_file);

			free((char *) path1);
			free((char *) path2);

			free((char *) coeff1gH);
			free((char *) coeff2gH);
			free((char *) Hpoint);

#if 0
			i = 0;

			free((char *) Hseq[i]);
			free((char *) Hname[i]);
			free((char *) Htitle[i]);
#endif
			sequ_seq_free(Hseq_seq);
		}

		V_seq_ftell = sequ_tell(query_file);
		sequ_rewind(pat_file);

		free((char *) Vpoint);
		free((char *) coeff1gV);
		free((char *) coeff2gV);
#if 0
		i = 0;
		free((char *) Vseq[i]);
		free((char *) Vname[i]);
		free((char *) Vtitle[i]);
#endif
		sequ_seq_free(Vseq_seq);
	}

#if 1
	free((char *) Vseq);
	free((char *) Vname);
	free((char *) Vtitle);

	free((char *) Hseq);
	free((char *) Hname);
	free((char *) Htitle);

	sequ_rewind(query_file);
#endif



	max_display = 20;
	max_display = MIN(max_display, num_scores);


	/* Print top alignments sorted by raw score ------------------ */

	qsort_raw_scores(record, num_scores);

	print_hist(bin_raw_score, foutp_raw);

	/* -- Generate alignments for sorted scores */
	/* get_alignment() calls print_alignment() */
	get_alignment(record, 
		      finp_pat, finp_query, foutp_raw, 
		      coeff1, coeff2, coeff3, coeff2A, S, AAseq, nAA, 
		      max_display, pat_file, query_file, 1 /* type = raw */);



	/* Print top alignments sorted by percent score ------------------ */

	qsort_percent_scores(record, num_scores);

	print_hist(bin_percent_score, foutp_percent);

	/* -- Generate alignments for sorted scores */
	/* get_alignment() calls print_alignment() */
	get_alignment(record, 
		      finp_pat, finp_query, foutp_percent, 
		      coeff1, coeff2, coeff3, coeff2A, S, AAseq, nAA, 
		      max_display, pat_file, query_file, 2 /* type = percent */);



	/* Print top alignments sorted by aligned score ------------------ */

	qsort_percent_align_scores(record, num_scores);

	print_hist(bin_percent_align_score, foutp_aligned);

	/* -- Generate alignments for sorted scores */
	/* get_alignment() calls print_alignment() */
	get_alignment(record, 
		      finp_pat, finp_query, foutp_aligned, 
		      coeff1, coeff2, coeff3, coeff2A, S, AAseq, nAA, 
		      max_display, pat_file, query_file, 3 /* type = aligned */);

	free((char *) coeff2A);

	fclose(finp_query);
	fclose(finp_pat);
	fclose(foutp_raw);
	fclose(foutp_percent);
	fclose(foutp_aligned);
	fclose(foutp_tmp);
/*	unlink(TMP_FILE_NAME);  */

	/* NOTREACHED */
	exit(0);
}

#if 0
void
free(void *p)
{
	return(0);
}
#endif
