/**************************************************************************
                      PIMA v. 1.40

  Randall F. Smith and Temple F. Smith (1991)
  Molecular Biology Computer Research Resource
  Galleria Level 1
  Dana-Farber Cancer Institute and School of Public Health
  Harvard University
  44 Binney St., Boston MA 02115 USA
  (617)732-3746
  INTERNET: rsmith@mbcrr.harvard.edu
  BITNET: rsmith%mbcrr@husc6.bitnet

  Copyright (c) 1989,1990,1991 MBCRR, Dana-Farber Cancer Institute and
                             Harvard University

*************************************************************************/

#include <pima.h>
#include <sequ.h>
char *program_name = "pima-pm";

void print_alignment(char (*C)[MAXAR], int *Vpoint, int *Hpoint, int **path1, int **path2, int len, FILE * foutp, char **Vseq, char **Hseq, int nV, int nH, int n_Vseqs, int n_Hseqs, char **Vname, char **Hname, char *Sname, char *Sext, sequ_format_t in_file_format);

int
main(int argc, char **argv)
{
    pima_params_t params[1];
    char **Vseq,
    **Hseq,
    **Vname,
    **Hname;
    char name[1000];
    float score;
    int Hmax;
    int *coeff1gH,
     *coeff2gH,
     *coeff1gV,
     *coeff2gV;
    int num_scores = 0;
    int nV, nH, n_Vseqs, n_Hseqs;
    int len;
    int *Vpoint,
     *Hpoint,
      start[3],
    **path1,
    **path2,
      l1,
      l2,
      i;
    long int *coeff2A;
    int sec_struct_seq;
    char *ext;
    int align_len;

    sequ_file_t *query_file,
     *pat_file;
    sequ_seq_t *Vseq_seq,
     *Hseq_seq;

/* 
 * *Vweight, *Hweight; FILE *finp_Vweight, *finp_Hweight; */
#ifdef UWP
    char alphabet[22];
    long bit_pattern[128],
     *Vprofile,
     *Hprofile;
    int is_a_seq;

#endif

#ifdef LOCAL_LOCAL
    int UPi,
      UPj,
      LOi,
      LOj,
      LOLOi,
      LOLOj;
    DIRECTION_VALUES **trace_back;
    int V_start,
      V_end,
      H_start,
      H_end;

#endif

#ifdef LOCAL_LOCAL
    UPi = UPj = LOi = LOj = LOLOi = LOLOj = -1;
#endif

    get_responses_pm(argc, argv, params);

    pat_file = sequ_new_read_first(&Vseq_seq, "pat", params->pattern_file);
    query_file = sequ_new_read_first(&Hseq_seq, "query", params->query_file);
    sequ_seq_free(Vseq_seq);
    sequ_seq_free(Hseq_seq);
    sequ_rewind(pat_file);
    sequ_rewind(query_file);

    /* max gap length = 128 (determined by the size of dist_north and
     * dist_west in DIRECTION_VALUES; see plsearch.h)
     */
    coeff2A = (long int *) calloc((unsigned) MAX(MAXV, MAXH) + 2,
				  sizeof(long int));

    check_calloc(coeff2A, "coeff2A");
    make_W2_array(params->coeff2, coeff2A, MAX(MAXV, MAXH), params->W2_atten);

#ifdef UWP
    strcpy(alphabet, "ACDEFGHIKLMNPQRSTVWY");
    len = strlen(alphabet);
    bit_pattern['X'] = 0L;	/* initialize */
    for (i = 0; i < len; i++) {
	bit_pattern[alphabet[i]] = octal[i + 1];
	bit_pattern['X'] = (bit_pattern['X'] | octal[i + 1]);
    }
    bit_pattern['B'] = (bit_pattern['D'] | bit_pattern['N']);
    bit_pattern['Z'] = (bit_pattern['E'] | bit_pattern['Q']);
    bit_pattern['g'] = (bit_pattern['X'] | octal[21]);
    bit_pattern['-'] = (bit_pattern['g'] | octal[22]);
#endif

    /* -- Read first input-file ---------------------------------
     * (file format = PAT-ID <tab> SEQUENCE)
     */

    n_Vseqs = 0;
    Vseq = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
    Vname = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));

    while ((Vseq_seq = sequ_read(pat_file)) != NULL) {

	len = Vseq_seq->length;
	Vseq[n_Vseqs] = (char *) calloc((unsigned) len + 2, sizeof(char));

	check_calloc((void *) Vseq[n_Vseqs], "Vseq in main()");
	strcpy(Vseq[n_Vseqs], Vseq_seq->seq);
	Vname[n_Vseqs] = (char *) calloc((unsigned) strlen(Vseq_seq->id) + 2,
					 sizeof(char));

	check_calloc((void *) Vname[n_Vseqs], "Vname in main()");
	strcpy(Vname[n_Vseqs], Vseq_seq->id);
	sequ_seq_free(Vseq_seq);

	if (n_Vseqs == 0) {
	    setup_seq(Vseq[0], Vname[0], params->AA_chars, params->n_AA,
		      params->coeff1, params->coeff2, &nV, &Vpoint, &coeff1gV, &coeff2gV);
	    /* Now that the actual sequence starts at 1, put a null at position
	     * zero so that when scanning from the end of the sequence towards
	     * the beginning for a non-dash character, if the sequence is all
	     * dashes the search will stop at a predictable place.
	     */
	    Vseq[0][0] = '\0';
#ifdef UWP
	    Vprofile = (long *) calloc((unsigned) len + 2, sizeof(long));

	    check_calloc((void *) Vprofile, "Vprofile in main()");
#endif
	} else {
	    /* Shift seq index by 1 so that seq[1] = seq[0], etc */
	    change_index(Vseq[n_Vseqs], &len);
	}

	sec_struct_seq = FALSE;
	if (params->sec_struct) {
	    strcpy(name, Vname[n_Vseqs]);
	    while ((ext = strchr(name, '.')) && !sec_struct_seq) {
		if (ext && *(ext + 1) == 's' && *(ext + 2) == 's')
		    sec_struct_seq = TRUE;
		strcpy(name, ext + 1);
	    }
	    if (sec_struct_seq) {
		for (i = 1; i <= len; i++) {
		    if (strchr(params->ss_chars, Vseq[n_Vseqs][i]))
			/* mark as being within ss unit by setting
			 * gap penalty > coeff1 */
			coeff1gV[i] = params->coeff1 + 1;
		}
	    }
	}
#ifdef UWP
	is_a_seq = TRUE;
	for (i = 1; i <= len; i++)
	    /* Note: 'g' removed from below */
	    if (strchr("abcdefhijklmnopqrstuvwxyz", Vseq[n_Vseqs][i])) {
		is_a_seq = FALSE;
		break;
	    }
	if (!sec_struct_seq && is_a_seq) {
	    /* then not a sec_struct or a pattern */
	    for (i = 1; i <= len; i++)
		Vprofile[i] = bit_pattern[Vseq[n_Vseqs][i]] | Vprofile[i];
	}
#endif

	n_Vseqs++;
    }


    /* -- Read second input-file ---------------------------------
     * (file format = PAT-ID <tab> SEQUENCE)
     */

    n_Hseqs = 0;
    Hseq = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));
    Hname = (char **) calloc((unsigned) MAX_SEQ, sizeof(char *));

    while ((Hseq_seq = sequ_read(query_file)) != NULL) {
	len = Hseq_seq->length;
	Hseq[n_Hseqs] = (char *) calloc((unsigned) len + 2, sizeof(char));

	check_calloc((void *) Hseq[n_Hseqs], "Hseq in main()");
	strcpy(Hseq[n_Hseqs], Hseq_seq->seq);
	Hname[n_Hseqs] = (char *) calloc((unsigned) strlen(Hseq_seq->id) + 2,
					 sizeof(char));

	check_calloc((void *) Hname[n_Hseqs], "Hname in main()");
	strcpy(Hname[n_Hseqs], Hseq_seq->id);
	sequ_seq_free(Hseq_seq);

	if (n_Hseqs == 0) {
	    setup_seq(Hseq[0], Hname[0], params->AA_chars, params->n_AA,
		      params->coeff1, params->coeff2, &nH, &Hpoint,
		      &coeff1gH, &coeff2gH);
	    /* See not above when setting Vseq[0][0]. */
	    Hseq[0][0] = '\0';
#ifdef UWP
	    Hprofile = (long *) calloc((unsigned) len + 2, sizeof(long));

	    check_calloc((void *) Hprofile, "Vprofile in main()");
#endif
	} else {
	    /* Shift seq index by 1 so that seq[1] = seq[0], etc */
	    change_index(Hseq[n_Hseqs], &len);
	}

	sec_struct_seq = FALSE;
	if (params->sec_struct) {
	    strcpy(name, Hname[n_Hseqs]);
	    while ((ext = strchr(name, '.')) && !sec_struct_seq) {
		if (ext && *(ext + 1) == 's' && *(ext + 2) == 's')
		    sec_struct_seq = TRUE;
		strcpy(name, ext + 1);
	    }
	    if (sec_struct_seq) {
		for (i = 1; i <= len; i++) {
		    if (strchr(params->ss_chars, Hseq[n_Hseqs][i]))
			/* mark as being within ss unit by setting
			 * gap penalty > coeff1 */
			coeff1gH[i] = params->coeff1 + 1;
		}
	    }
	}
#ifdef UWP
	is_a_seq = TRUE;
	for (i = 1; i <= len; i++)
	    /* Note: 'g' removed from below */
	    if (strchr("abcdefhijklmnopqrstuvwxyz", Hseq[n_Hseqs][i])) {
		is_a_seq = FALSE;
		break;
	    }
	if (!sec_struct_seq && is_a_seq) {
	    /* then not a sec_struct or a pattern */
	    for (i = 1; i <= len; i++)
		Hprofile[i] = bit_pattern[Hseq[n_Hseqs][i]] | Hprofile[i];
	}
#endif

	n_Hseqs++;
    }


/* 
 * Vweight = (int *) calloc ( (unsigned) nV+2, sizeof(int));
 * check_calloc((void *) Vweight, "Vweight in main()");                   for
 * (i=1; i<= nV; i++) fscanf(finp_Vweight, "%d", Vweight[i]);
 * 
 * Hweight = (int *) calloc ( (unsigned) nH+2, sizeof(int));
 * check_calloc((void *) Hweight, "Hweight in main()");                   for
 * (i=1; i<= nH; i++) fscanf(finp_Hweight, "%d", Hweight[i]); */


    /* Find the best match score (Hmax)  */

    path1 = (int **) calloc((unsigned) 2, sizeof(int *));

    check_calloc((void *) path1, "path1");
    path2 = (int **) calloc((unsigned) 2, sizeof(int *));

    check_calloc((void *) path2, "path2");
    for (i = 0; i < 2; i++) {
	path1[i] = (int *) calloc((unsigned) nV + nH, sizeof(int));

	check_calloc(path1[i], "path1[i]");
	path2[i] = (int *) calloc((unsigned) nV + nH, sizeof(int));

	check_calloc(path2[i], "path2[i]");
    }

#ifdef LOCAL_LOCAL
    V_start = 1;
    while (Vseq[0][V_start] == '-')
	V_start++;
    V_end = nV;
    while (Vseq[0][V_end] == '-')
	V_end--;
    H_start = 1;
    while (Hseq[0][H_start] == '-')
	H_start++;
    H_end = nH;
    while (Hseq[0][H_end] == '-')
	H_end--;


    /* See if either sequence is all dashes, in that case just call
     * print_alignment with a len of zero.
     */
    if (V_end == 0 || H_end == 0) {
	printf("\n0\n");	/* Score is zero. */
	print_alignment(params->C, Vpoint, Hpoint, path1, path2, 0,
			params->root_file, Vseq, Hseq, nV, nH,
			n_Vseqs, n_Hseqs, Vname, Hname, params->pattern_name,
			params->name_suffix, query_file->format);
	/* Might consider freeing memory and closing files. */
	exit(0);
    }

    trace_back = (DIRECTION_VALUES **) calloc((unsigned)
			 (V_end - V_start + 2), sizeof(DIRECTION_VALUES *));
    check_calloc((int *) trace_back, "trace_back");

    for (i = 0; i <= V_end - V_start + 1; i++) {
	trace_back[i] = (DIRECTION_VALUES *) calloc((unsigned)
						    (H_end - H_start + 2),
						  sizeof(DIRECTION_VALUES));
	check_calloc((void *) trace_back[i], "trace_back[i]");
    }
#endif


    paths(Vpoint, Hpoint, path1, &l1, path2, &l2,
	  nV, nH, params->S, start, coeff2A, &Hmax,
	  coeff1gV, coeff2gV, coeff1gH, coeff2gH,
	  params->coeff1, params->coeff3, Vseq[0], Hseq[0], FALSE, &align_len
#ifdef UWP
	  ,Vprofile, Hprofile
#endif
#ifdef LOCAL_LOCAL
	  ,UPi, UPj, LOi, LOj, &LOLOi, &LOLOj,
	  trace_back, V_start, V_end, H_start, H_end,
	  params->min_local_score, params->min_local_width
#endif
	);

    score = (float) Hmax / 100.0;
    num_scores++;

    fprintf(stdout, "\n%g\n", score);
/* fprintf(stdout, "\n%g\t%g\n", score, score/align_len); */
    print_alignment(params->C, Vpoint, Hpoint, path1, path2,
		    l1, params->root_file, Vseq, Hseq, nV, nH,
		    n_Vseqs, n_Hseqs, Vname, Hname,
		    params->pattern_name, params->name_suffix,
		    query_file->format);


    for (i = 0; i < 2; i++) {
	free((char *) path1[i]);
	free((char *) path2[i]);
    }
    free((char *) path1);
    free((char *) path2);

    free((char *) coeff2A);

    free((char *) Hpoint);
    free((char *) coeff1gH);
    free((char *) coeff2gH);
    free((char *) coeff1gV);
    free((char *) coeff2gV);
    free((char *) Vpoint);
    for (i = 0; i < n_Hseqs; i++) {
	free((char *) Hseq[i]);
	free((char *) Hname[i]);
    }
    for (i = 0; i < n_Vseqs; i++) {
	free((char *) Vseq[i]);
	free((char *) Vname[i]);
    }


#ifdef UWP
    free((char *) Vprofile);
    free((char *) Hprofile);
#endif

#ifdef LOCAL_LOCAL
    for (i = 0; i <= V_end - V_start + 1; i++)
	free((char *) trace_back[i]);
    free((char *) trace_back);
#endif


    fclose(params->query_file);
    fclose(params->pattern_file);
    fclose(params->root_file);

    /* NOTREACHED */
    exit(0);
}
