#define MAX_ID_LEN 2000

/***************************************************************************/
#include <pima.h>
#include <memu_macros.h>
#include <sequ.h>

void print_seqs(char *Spath, char **Vpath, char **Hpath, int count, int n_Vseqs, int n_Hseqs, FILE *foutp, char **Vname, char **Hname, char *Sname, char *Sext, sequ_format_t in_file_format);

void
print_alignment(char (*C)[MAXAR], int *Vpoint, int *Hpoint, int **path1, int **path2, int len, FILE *foutp, char **Vseq, char
		**Hseq, int nV, int nH, int n_Vseqs, int n_Hseqs, char **Vname, char **Hname, char *Sname, char *Sext, sequ_format_t in_file_format)
{
    int i0,
    i1,
    i2,
    j0,
    j1,
    j2;
    int j,
    k,
    m,
    count,
    numgaps;
    int Hstart,
    Vstart,
    count2;
    int gap_1,
    j00,
    j01;
    char **Vpath,
    **Hpath,
    *Spath;

    Vpath = (char **) calloc((unsigned) (n_Vseqs + 1), sizeof(char *));
    check_calloc((void *) Vpath, "Vpath");
    for (k = 0; k < n_Vseqs; k++) {
	Vpath[k] = (char *) calloc((unsigned) (nV + nH + 1), sizeof(char));
	check_calloc((void *) Vpath[k], "Vpath[k]");
    }

    Hpath = (char **) calloc((unsigned) (n_Hseqs + 1), sizeof(char *));
    check_calloc((void *) Hpath, "Hpath");
    for (k = 0; k < n_Hseqs; k++) {
	Hpath[k] = (char *) calloc((unsigned) (nV + nH + 1), sizeof(char));
	check_calloc((void *) Hpath[k], "Hpath[k]");
    }
    Spath = (char *) calloc((unsigned) (nV + nH + 1), sizeof(char));
    check_calloc((void *) Spath, "Spath");

    /* Check if this is a null alignment, in that case just print a pattern
     * with all dashes and the sequences side by side.
     */
    if (len < 1) {
	count = 1;
	for (j = nH; j > 0; j--, count++) {
	    Spath[count] = '-';
	    for (k = 0; k < n_Hseqs; k++) {
		Hpath[k][count] = Hseq[k][j];
	    }
	    for (k = 0; k < n_Vseqs; k++) {
		Vpath[k][count] = '-';
	    }
	}
	for (j = nV; j > 0; j--, count++) {
	    Spath[count] = '-';
	    for (k = 0; k < n_Hseqs; k++) {
		Hpath[k][count] = '-';
	    }
	    for (k = 0; k < n_Vseqs; k++) {
		Vpath[k][count] = Vseq[k][j];
	    }
	}
	count--;
	print_seqs(Spath, Vpath, Hpath, count, n_Vseqs, n_Hseqs, foutp, Vname, Hname, Sname, Sext, in_file_format);
	return;
    }

    Hstart = path1[1][1] + 1;
    Vstart = path1[0][1] + 1;

    count = 1;
    if ((nH - Hstart) >= (nV - Vstart)) {
	for (j = nH; j > Hstart; j--) {
	    for (k = 0; k < n_Hseqs; k++)
		Hpath[k][count] = Hseq[k][j];
	    Spath[count] = '-';
	    for (k = 0; k < n_Vseqs; k++)
		Vpath[k][count] = '-';
	    count++;
	}
	count2 = count - 1;
	for (j = Vstart + 1; j <= nV; j++) {
	    for (k = 0; k < n_Vseqs; k++)
		Vpath[k][count2] = Vseq[k][j];
	    count2--;
	}
    } else {
	for (j = nV; j > Vstart; j--) {
	    for (k = 0; k < n_Vseqs; k++)
		Vpath[k][count] = Vseq[k][j];
	    Spath[count] = '-';
	    for (k = 0; k < n_Hseqs; k++)
		Hpath[k][count] = '-';
	    count++;
	}
	count2 = count - 1;
	for (j = Hstart + 1; j <= nH; j++) {
	    for (k = 0; k < n_Hseqs; k++)
		Hpath[k][count2] = Hseq[k][j];
	    count2--;
	}
    }

    i0 = path1[0][1] + 2;
    j0 = path1[1][1] + 2;

    /* Set these just to avoid the uninitialized variable warning, we know
     * len is greater than zero so these values will be assigned correctly,
     * but the compiler considers the loop conditional and will complain
     * because the values are used after the loop.
     */
    i1 = 0;
    j1 = 0;

    for (j = 1; j <= len; j++) {
	i1 = path1[0][j] + 1;
	j1 = path1[1][j] + 1;
	i2 = path2[0][j] + 1;
	j2 = path2[1][j] + 1;
	if ((i1 == i2) && (j1 == j2)) {
	    if (i1 == i0) {
		numgaps = (j0 - j1);
		for (k = 0; k < numgaps; k++) {
		    for (m = 0; m < n_Vseqs; m++)
			Vpath[m][count] = '-';
		    for (m = 0; m < n_Hseqs; m++)
			Hpath[m][count] = Hseq[m][j0 - 1 - k];
		    Spath[count] = 'g';
		    count++;
		}
	    } else if (j1 == j0) {
		numgaps = (i0 - i1);
		for (k = 0; k < numgaps; k++) {
		    for (m = 0; m < n_Hseqs; m++)
			Hpath[m][count] = '-';
		    for (m = 0; m < n_Vseqs; m++)
			Vpath[m][count] = Vseq[m][i0 - 1 - k];
		    Spath[count] = 'g';
		    count++;
		}
	    } else {
		for (m = 0; m < n_Vseqs; m++)
		    Vpath[m][count] = Vseq[m][i1];
		for (m = 0; m < n_Hseqs; m++)
		    Hpath[m][count] = Hseq[m][j1];
		Spath[count] = C[Vpoint[i1]][Hpoint[j1]];
		count++;
	    }
	    i0 = i1;
	    j0 = j1;

	} else {
	    if ((i1 == i0) || (j1 == j0))
		gap_1 = 0;	/* gap starts in path2 */
	    else
		gap_1 = 1;	/* gap starts in path1 */
	    j00 = j01 = j;
	    while (!((i1 == i2) && (j1 == j2))) {
		j01++;
		i1 = path1[0][j01] + 1;
		j1 = path1[1][j01] + 1;
		i2 = path2[0][j01] + 1;
		j2 = path2[1][j01] + 1;
	    }
	    for (j = j00; j < j01; j++) {
		if (gap_1) {
		    i1 = path1[0][j] + 1;
		    j1 = path1[1][j] + 1;
		} else {
		    i1 = path2[0][j] + 1;
		    j1 = path2[1][j] + 1;
		}
		if (i1 == i0) {
		    numgaps = (j0 - j1);
		    for (k = 0; k < numgaps; k++) {
			for (m = 0; m < n_Vseqs; m++)
			    Vpath[m][count] = '-';
			for (m = 0; m < n_Hseqs; m++)
			    Hpath[m][count] = Hseq[m][j0 - 1 - k];
			Spath[count] = 'g';
			count++;
		    }
		} else if (j1 == j0) {
		    numgaps = (i0 - i1);
		    for (k = 0; k < numgaps; k++) {
			for (m = 0; m < n_Hseqs; m++)
			    Hpath[m][count] = '-';
			for (m = 0; m < n_Vseqs; m++)
			    Vpath[m][count] = Vseq[m][i0 - 1 - k];
			Spath[count] = 'g';
			count++;
		    }
		} else {
		    for (m = 0; m < n_Vseqs; m++)
			Vpath[m][count] = Vseq[m][i1];
		    for (m = 0; m < n_Hseqs; m++)
			Hpath[m][count] = Hseq[m][j1];
		    Spath[count] = C[Vpoint[i1]][Hpoint[j1]];
		    /* 
		     * if (Spath[count] != 'g') Spath[count] = 'x'; */
		    count++;
		}
		i0 = i1;
		j0 = j1;
	    }
	    j--;
	}
    }

    if (i1 >= j1) {
	count2 = count;
	for (j = i1 - 1; j > 0; j--) {
	    for (k = 0; k < n_Vseqs; k++)
		Vpath[k][count] = Vseq[k][j];
	    Spath[count] = '-';
	    for (k = 0; k < n_Hseqs; k++)
		Hpath[k][count] = '-';
	    count++;
	}
	for (j = j1 - 1; j > 0; j--) {
	    for (k = 0; k < n_Hseqs; k++)
		Hpath[k][count2] = Hseq[k][j];
	    Spath[count2] = '-';
	    count2++;
	}
    } else {
	count2 = count;
	for (j = j1 - 1; j > 0; j--) {
	    for (k = 0; k < n_Hseqs; k++)
		Hpath[k][count] = Hseq[k][j];
	    Spath[count] = '-';
	    for (k = 0; k < n_Vseqs; k++)
		Vpath[k][count] = '-';
	    count++;
	}
	for (j = i1 - 1; j > 0; j--) {
	    for (k = 0; k < n_Vseqs; k++)
		Vpath[k][count2] = Vseq[k][j];
	    Spath[count2] = '-';
	    count2++;
	}
    }
    count--;

#if 0
    /* These ones at the end of the sequences are no longer needed. */
    for (k = 0; k < n_Vseqs; k++)
	Vpath[k][0] = '1';
    for (k = 0; k < n_Hseqs; k++)
	Hpath[k][0] = '1';
    Spath[0] = '1';
#endif

    /*  strip X's and g's from both ends of the pattern  */
    k = 1;
    while (k <= count && strchr("-Xg", Spath[k])) {
	Spath[k] = '-';
	k++;
    }
    k = count;
    while (k > 0 && strchr("-Xg", Spath[k])) {
	Spath[k] = '-';
	k--;
    }
    print_seqs(Spath, Vpath, Hpath, count, n_Vseqs, n_Hseqs, foutp, Vname, Hname, Sname, Sext, in_file_format);
}


void
print_seqs(char *Spath, char **Vpath, char **Hpath, int count, int n_Vseqs, int n_Hseqs, FILE *foutp, char **Vname, char **Hname, char *Sname, char *Sext, sequ_format_t in_file_format)
{
    int j;
    int k;
    sequ_file_t	*out_file;
    sequ_seq_t	out_seq[1];
    char		*p;

    out_file = sequ_new_write("output", foutp, in_file_format);
    memu_new_m(out_seq->id, char, MAX_ID_LEN);
    memu_new_m(out_seq->seq, char, count + 1);
    out_seq->seq[count] = '\0';
    out_seq->length = count;
    out_seq->title = NULL;

    sprintf(out_seq->id, "%s%s", Sname, Sext);
    for (p = out_seq->seq, j = count; j >= 1; j--)
	*p++ = Spath[j];
    sequ_write(out_file, out_seq);

    for (k = 0; k < n_Vseqs; k++) {
	sprintf(out_seq->id, "%s%s", Vname[k], Sext);
	for (p = out_seq->seq, j = count; j >= 1; j--)
	    *p++ = Vpath[k][j];
	sequ_write(out_file, out_seq);
	fflush(foutp);
    }

    for (k = 0; k < n_Hseqs; k++) {
	sprintf(out_seq->id, "%s%s", Hname[k], Sext);
	for (p = out_seq->seq, j = count; j >= 1; j--)
	    *p++ = Hpath[k][j];
	sequ_write(out_file, out_seq);
	fflush(foutp);
    }

    fflush(foutp);

    memu_free_m(out_seq->id);
    memu_free_m(out_seq->seq);

    for (k = 0; k < n_Vseqs; k++)
	free((char *) Vpath[k]);
    for (k = 0; k < n_Hseqs; k++)
	free((char *) Hpath[k]);
    free((char *) Vpath);
    free((char *) Hpath);
    free((char *) Spath);
}
