/*
 * extract-root-pat.c - Extract root pattern sequence from PIMA file.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <config.h>
#include <sequ.h>
#include <memu_macros.h>

char *program_name = "extract-root-pat";
void print_help();

int
main(int argc, char **argv)
{
    int		c;
    /* extern char *	optarg; */
    extern int	optind;
    int errorFlag = 0;

    sequ_file_t *in_file;
    sequ_file_t *out_file = NULL;
    sequ_seq_t	*in_seq;
    char	*new_seq_seq;
    char	*old_seq_seq;
    char	*n;
    char	*o;

    while((c = getopt(argc, argv, "h")) != EOF) {
	switch(c) {
	case 'h':
	    print_help();
	    exit(0);
	default:
	    errorFlag++;
	    break;
	}
    }

    /* Should be atleast one file. */
    if (optind >= argc) {
	errorFlag++;
    }
	
    if (errorFlag) {
	fprintf(stderr,
		"Usage: %s [pima-files]\n"
		"   or: %s -h\n", program_name, program_name);
		exit(2);
    }

    while (optind < argc) {
	in_file = sequ_new_read_first(&in_seq, argv[optind], NULL);
	if (out_file == NULL) {
	    out_file = sequ_new_write("stdout", stdout, in_file->format);
	}

	/* Make sure there is atleast one sequence. */
	if (in_seq != NULL) {
	    /* Make room for new copy of input sequence sequence. */
	    memu_new_m(new_seq_seq, char, in_seq->length);
	    o = in_seq->seq;
	    n = new_seq_seq;

	    /* Skip over leading dashes. */
	    while (*o && *o == '-') {
		o++;
	    }

	    /* Copy until end of input */
	    while (*o) {
		*n++ = *o++;
	    }
	    *n = '\0';

	    /* Strip off trailing dashes from new sequence */
	    n--;
	    while (n >= new_seq_seq && *n == '-') {
		*n-- = '\0';
	    }

	    /* Just in case there was no sequence left. */
	    if (n < new_seq_seq) {
		n = new_seq_seq;
	    }

	    /* Just switch the two sequences. */
	    old_seq_seq = in_seq->seq;
	    in_seq->seq = new_seq_seq;
	    in_seq->length = n - new_seq_seq;

	    /* Write out the new sequence. */
	    sequ_write(out_file, in_seq);

	    /* Free up the memory used. */
	    sequ_seq_free(in_seq);
	    memu_free_m(old_seq_seq);
	}

	/* Close the input sequence file, not the output. */
	sequ_close(in_file);
	optind++;
    }

    sequ_close(out_file);
    exit(0);
}

void
print_help()
{
    printf("%s: help\n", program_name);
    printf("     Extracts the final root pattern from one or more\n");
    printf("'CLUSTER_NAME.pima' files created by the program 'pima'.\n");
}
