/*
 * trim-root-num.c - Trim root pattern number from pima file.
 *
 * Copyright (C) 1994 Baylor College of Medicine
 *
 * All rights reserved.
 *
 * Contact Brent Wiese, brent@bcm.tmc.edu
 */

#include <config.h>
#include <sequ.h>

char *program_name = "trim-root-num";
void print_help();

int
main(int argc, char **argv)
{
    int		c;
    /* extern char *	optarg; */
    extern int	optind;
    int errorFlag = 0;

    sequ_file_t *in_file;
    sequ_file_t *out_file;
    sequ_seq_t	*in_seq;
    char 	*p;

    while((c = getopt(argc, argv, "h")) != EOF) {
	switch(c) {
	case 'h':
	    print_help();
	    exit(0);
	default:
	    errorFlag++;
	    break;
	}
    }

    /* Should be only one file. */
    if (optind + 1 != argc) {
	errorFlag++;
    }
	
    if (errorFlag) {
	fprintf(stderr,
		"Usage: %s pima-file\n"
		"   or: %s -h\n", program_name, program_name);
		exit(2);
    }

    in_file = sequ_new_read_first(&in_seq, argv[optind], NULL);
    out_file = sequ_new_write("stdout", stdout, in_file->format);

    while (in_seq != NULL) {
	p = strrchr(in_seq->id, '.');
	if (p == NULL) {
	    fprintf(stderr, "%s: missing trailing period.\n", program_name);
	    exit(1);
	}
	*p = '\0';

	/* Write out the new sequence. */
	sequ_write(out_file, in_seq);
	
	/* Free up the memory used. */
	sequ_seq_free(in_seq);
	
	/* Possibly read the next sequence. */
	in_seq = sequ_read(in_file);
    }

	/* Close the input sequence file, not the output. */
    sequ_close(in_file);
    sequ_close(out_file);
    exit(0);
}

void
print_help()
{
    printf("%s: help\n", program_name);
    printf("     Extracts the final root pattern from one or more\n");
    printf("'CLUSTER_NAME.pima' files created by the program 'pima'.\n");
}
